@@ -49,7 +49,6 @@ import asyncio
 import concurrent.futures
 import dataclasses
 from typing import Any, Coroutine, Dict, List, Mapping, MutableSet, Optional, Tuple, Type
-import uuid
 
 from absl import logging
 from etils import epath
@@ -504,7 +503,6 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
     self._async_options = (
         composite_options.async_options or options_lib.AsyncOptions()
     )
-
     logging.info(
         'Initialized registry %s.',
         self._handler_registry,
@@ -669,7 +667,6 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
 
     return result
 
-
   async def async_save(
       self, directory: epath.Path, args: CompositeArgs
   ) -> Optional[List[Future]]:
@@ -748,11 +745,11 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
 
     Restoration can happen in a variety of modes, depending on which `args` is
     passed:
-      - `args` is not None and not empty. Item names present in `args` will
-        be restored as long as they exist in the checkpoint and have a
-        registered handler.
+      - `args` is not None and not empty. Item names present in `args` will be
+      restored as long as they exist in the checkpoint and have a registered
+      handler.
       - `args` is None or empty. All items in the checkpoint that have a
-        registered handler will be restored.
+      registered handler will be restored.
 
     Args:
       directory: Path to restore from.
@@ -1012,7 +1009,8 @@ class CompositeCheckpointHandler(AsyncCheckpointHandler):
         # Not an error, as some items may not have been saved.
         continue
       handler.finalize(tmp_dir.get())
-      tmp_dir.finalize()
+      tmp_dir.finalize(
+      )
 
       # Remove the temporary path once it has been finalized.
       self._current_temporary_paths.pop(item_name)
