@@ -691,12 +691,11 @@ class PyTreeCheckpointHandlerTestBase:
     def test_cast_scalar(self):
       pytree = {'a': 5, 'b': 1.2}
       restore_args = {
-          # Can restore as int and later cast to float.
           'a': RestoreArgs(
-              dtype=np.float32, restore_type=int
+              restore_type=float
           ),  # pytype: disable=wrong-arg-types  # jnp-type
           'b': RestoreArgs(
-              dtype=np.float64, restore_type=float
+              restore_type=int
           ),  # pytype: disable=wrong-arg-types  # jnp-type
       }
 
@@ -705,7 +704,7 @@ class PyTreeCheckpointHandlerTestBase:
           self.directory, args=PyTreeRestoreArgs(restore_args=restore_args)
       )
       self.assertIsInstance(restored['a'], float)
-      self.assertIsInstance(restored['b'], float)
+      self.assertIsInstance(restored['b'], int)
 
     def test_restore_type(self):
       pytree = {'a': 5, 'b': 6.1}
@@ -1515,7 +1514,12 @@ class PyTreeCheckpointHandlerTestBase:
       axes = jax.sharding.PartitionSpec(
           'x',
       )
-      tree = {'x': test_utils.create_sharded_array(np.arange(8), mesh, axes)}
+      dtype = np.float32
+      tree = {
+          'x': test_utils.create_sharded_array(
+              np.arange(8, dtype=dtype), mesh, axes
+          )
+      }
       restore_args = {
           'x': ArrayRestoreArgs(
               mesh=mesh, mesh_axes=axes, global_shape=(16,), strict=strict
@@ -1533,7 +1537,11 @@ class PyTreeCheckpointHandlerTestBase:
         )
         expected = {
             'x': test_utils.create_sharded_array(
-                np.concatenate((np.arange(8), np.zeros(8))), mesh, axes
+                np.concatenate(
+                    (np.arange(8, dtype=dtype), np.zeros(8, dtype=dtype))
+                ),
+                mesh,
+                axes,
             )
         }
         self.validate_restore(expected, restored)
@@ -2145,7 +2153,7 @@ class PyTreeCheckpointHandlerTestBase:
         )
 
         expected_tree_with_write_shapes = {
-            'a': {'write_shape': (8,)},
+            'a': {'write_shape': (1,)},
             'b': {'write_shape': (2,)},
             'c': {
                 'a': {'write_shape': (1, 1)},
@@ -2204,12 +2212,11 @@ class PyTreeCheckpointHandlerTestBase:
       if multihost.process_index() == 0:
         array_metadatas = await ARRAY_METADATA_STORE.read(self.directory)
         expected_array_metadatas = {
-            # Expected with use_replica_parallel=False.
             0: [
                 array_metadata.SerializedArrayMetadata(
                     param_name='new_key.a',
-                    write_shape=(8,),
-                    chunk_shape=(8,),
+                    write_shape=(1,),
+                    chunk_shape=(1,),
                 ),
                 array_metadata.SerializedArrayMetadata(
                     param_name='new_key.b',
@@ -2230,8 +2237,8 @@ class PyTreeCheckpointHandlerTestBase:
             1: [
                 array_metadata.SerializedArrayMetadata(
                     param_name='new_key.a',
-                    write_shape=(8,),
-                    chunk_shape=(8,),
+                    write_shape=(1,),
+                    chunk_shape=(1,),
                 ),
                 array_metadata.SerializedArrayMetadata(
                     param_name='new_key.b',
@@ -2319,3 +2326,55 @@ class PyTreeCheckpointHandlerTestBase:
       tree = [arr]
       with self.assertRaisesRegex(ValueError, 'zero size'):
         self.handler.save(self.directory, args=PyTreeSaveArgs(tree))
+
+    @parameterized.product(use_ocdbt=(True, False))
+    def test_save_restore_random_keys(self, use_ocdbt: bool):
+      """Test saving and restoring random keys within a pytree."""
+
+      # TODO(b/393160483) investigate Pathways remote Python support for
+      # random.keys.
+      if utils.is_pathways_backend():
+        self.skipTest(
+            'Disabled on Pathways because random keys are not supported by'
+            ' remote Python.'
+        )
+
+      mesh = jax.sharding.Mesh(jax.devices(), ('x',))
+      sharding = jax.sharding.NamedSharding(mesh, jax.sharding.PartitionSpec())
+
+      pytree = {
+          'keys': {
+              'kone': jax.random.key(jnp.array(0, device=sharding)),
+              'impl_key': {
+                  'rbg': jax.random.key(
+                      jnp.array(1, device=sharding), impl='rbg'
+                  ),
+                  'unsafe_rbg': jax.random.key(
+                      jnp.array(2, device=sharding), impl='unsafe_rbg'
+                  ),
+              },
+              'split_keys': jax.random.split(
+                  jax.random.key(jnp.array(123, device=sharding)), num=10
+              ),
+          },
+          'arrays': self.pytree,
+      }
+
+      restore_args = jax.tree.map(
+          lambda x: ArrayRestoreArgs(sharding=x.sharding), pytree
+      )
+
+      with self.ocdbt_checkpoint_handler(
+          use_ocdbt=use_ocdbt,
+          array_metadata_store=array_metadata_store_lib.Store(),
+      ) as save_handler:
+        save_handler.save(self.directory, args=PyTreeSaveArgs(pytree))
+
+      with self.ocdbt_checkpoint_handler(
+          use_ocdbt=use_ocdbt,
+          array_metadata_store=array_metadata_store_lib.Store(),
+      ) as restore_handler:
+        restored = restore_handler.restore(
+            self.directory, args=PyTreeRestoreArgs(pytree, restore_args)
+        )
+        test_utils.assert_tree_equal(self, pytree, restored)
