@@ -25,12 +25,14 @@ from etils import epath
 import jax
 from orbax.checkpoint import checkpoint_args
 from orbax.checkpoint import checkpoint_utils
-from orbax.checkpoint import future
 from orbax.checkpoint import options as options_lib
 from orbax.checkpoint._src import asyncio_utils
+from orbax.checkpoint._src.futures import future
 from orbax.checkpoint._src.handlers import async_checkpoint_handler
 from orbax.checkpoint._src.handlers import pytree_checkpoint_handler
 from orbax.checkpoint._src.metadata import pytree_metadata_options as pytree_metadata_options_lib
+from orbax.checkpoint._src.metadata import tree as tree_metadata
+from orbax.checkpoint._src.tree import types as tree_types
 from orbax.checkpoint._src.tree import utils as tree_utils
 
 
@@ -143,15 +145,19 @@ class StandardCheckpointHandler(
           'Make sure to specify kwarg name `args=` when providing'
           ' `StandardSaveArgs`.'
       )
+    custom_metadata = None
     if args is not None:
       item = args.item
       save_args = args.save_args
+      custom_metadata = args.custom_metadata
 
     self._validate_save_state(item, save_args=save_args)
     return await self._impl.async_save(
         directory,
         args=pytree_checkpoint_handler.PyTreeSaveArgs(
-            item=item, save_args=save_args
+            item=item,
+            save_args=save_args,
+            custom_metadata=custom_metadata,
         ),
     )
 
@@ -212,7 +218,9 @@ class StandardCheckpointHandler(
       args = StandardRestoreArgs(item=item)
     if args.item is not None:
       self._validate_restore_state(args.item)
-      restore_args = checkpoint_utils.construct_restore_args(args.item)
+      restore_args = checkpoint_utils.construct_restore_args(
+          args.item, support_layout=args.support_layout
+      )
     else:
       logging.warning(
           '`StandardCheckpointHandler` expects a target tree to be provided for'
@@ -240,7 +248,7 @@ class StandardCheckpointHandler(
         ),
     )
 
-  def metadata(self, directory: epath.Path) -> PyTree:
+  def metadata(self, directory: epath.Path) -> tree_metadata.TreeMetadata:
     """Returns metadata about the saved item."""
     return self._impl.metadata(directory)
 
@@ -263,10 +271,18 @@ class StandardSaveArgs(CheckpointArgs):
     save_args: a PyTree with the same structure of `item`, which consists of
       `ocp.SaveArgs` objects as values. `None` can be used for values where no
       `SaveArgs` are specified.
+    custom_metadata: User-provided custom metadata. An arbitrary
+      JSON-serializable dictionary the user can use to store additional
+      information. The field is treated as opaque by Orbax.
   """
 
   item: PyTree
   save_args: Optional[PyTree] = None
+  custom_metadata: tree_types.JsonType | None = None
+
+  def __post_init__(self):
+    if isinstance(self.item, tree_metadata.TreeMetadata):
+      raise ValueError('Cannot save TreeMetadata.')
 
 
 @register_with_handler(StandardCheckpointHandler, for_restore=True)
@@ -278,13 +294,17 @@ class StandardRestoreArgs(CheckpointArgs):
 
   Attributes (all optional):
     item: target PyTree. Currently non-optional. Values may be either real
-        array or scalar values, or they may be jax.ShapeDtypeStruct. If real
-        values are provided, that value will be restored as the given type, with
+        array or scalar values, or they may be jax.ShapeDtypeStruct, or
+        `ocp.metadata.value.Metadata` objects (which come from calling the
+        `metadata` method). If real values are provided,
+        that value will be restored as the given type, with
         the given properties. If jax.ShapeDtypeStruct is provided, the value
         will be restored as np.ndarray, unless `sharding` is specified. If
         `item` is a custom PyTree class, the tree will be restored with the
         same structure as provided. If not provided, restores as a serialized
         nested dict representation of the custom class.
+        `TreeMetadata` is also allowed as the tree used to
+        define the restored structure.
     strict: if False, restoration allows silent truncating/padding of arrays if
         the stored array shape does not match the target shape. Otherwise,
         raises an error.
@@ -294,3 +314,7 @@ class StandardRestoreArgs(CheckpointArgs):
   item: Optional[PyTree] = None
   strict: bool = True
   support_layout: bool = False
+
+  def __post_init__(self):
+    if isinstance(self.item, tree_metadata.TreeMetadata):
+      self.item = self.item.tree
