@@ -91,6 +91,7 @@ class SerDeserializer:
             'param_name': array_metadata.param_name,
             'write_shape': array_metadata.write_shape,
             'chunk_shape': array_metadata.chunk_shape,
+            'ext_metadata': array_metadata.ext_metadata,
         }
     }
 
@@ -102,6 +103,7 @@ class SerDeserializer:
           param_name=array_metadata['param_name'],
           write_shape=tuple(array_metadata['write_shape']),
           chunk_shape=tuple(array_metadata['chunk_shape']),
+          ext_metadata=array_metadata.get('ext_metadata'),
       )
     return obj
 
@@ -131,9 +133,45 @@ class Store:
       self,
       path_resolver: PathResolver = PathResolver(),
       ser_deser: SerDeserializer = SerDeserializer(),
+      primary_host: int | None = 0,  # None means all hosts are primary hosts.
+      write_timeout_secs: int = 600,  # 10 minutes.
   ):
     self._path_resolver = path_resolver
     self._ser_deser = ser_deser
+    self._primary_host = primary_host
+    self._write_timeout_secs = write_timeout_secs
+
+  def set_primary_host(self, primary_host: int | None) -> None:
+    """Sets the primary host."""
+    self._primary_host = primary_host
+
+  async def _maybe_create_base_dir(self, base_dir: epath.Path) -> None:
+    """Primary host creates the base directory, rest of the hosts wait."""
+    if multihost.is_primary_host(self._primary_host):
+      # primary host creates, rest of the hosts wait.
+      return await asyncio.to_thread(
+          base_dir.mkdir, parents=True, exist_ok=True
+      )
+
+    # non-primary host waits for primary host to create the base dir/folder.
+    async def wait_for_base_dir_creation():
+      while not await asyncio.to_thread(base_dir.exists):
+        await asyncio.sleep(0.25)
+
+    try:
+      await asyncio.wait_for(
+          wait_for_base_dir_creation(), timeout=self._write_timeout_secs
+      )
+    except asyncio.TimeoutError as e:
+      primary_process = (
+          'LOCAL' if self._primary_host is None else self._primary_host
+      )
+      raise ValueError(
+          f'[process_index={multihost.process_index()}] Timed out waiting for'
+          f' array_metadatas base directory creation: {base_dir}.'
+          f' timeout={self._write_timeout_secs} seconds.'
+          f' primary_process={primary_process}'
+      ) from e
 
   async def write(
       self,
@@ -153,7 +191,7 @@ class Store:
     file_path = self._path_resolver.get_write_file_path(
         checkpoint_dir, process_index
     )
-    await asyncio.to_thread(file_path.parent.mkdir, parents=True, exist_ok=True)
+    await self._maybe_create_base_dir(file_path.parent)
     await asyncio.to_thread(
         file_path.write_text, self._ser_deser.serialize(array_metadatas)
     )
@@ -357,8 +395,9 @@ class Validator:
           process_index=process_index,
           process_cache=process_cache,
       )
-      # Check if the chunk_shape and write_shape are the same for each param.
-      self._validate_chunk_shape_and_write_shape(
+      # Check if the chunk_shape, write_shape and ext_metadata are the same for
+      # each param.
+      self._validate_metadata(
           ref_process_index=ref_process_index,
           ref_process_cache=ref_process_cache,
           process_index=process_index,
@@ -449,7 +488,7 @@ class Validator:
           f' {len(missing_in_ref_process)} params: {missing_in_ref_process}.'
       )
 
-  def _validate_chunk_shape_and_write_shape(
+  def _validate_metadata(
       self,
       *,
       ref_process_index: int,
@@ -477,6 +516,40 @@ class Validator:
             f' {ref_array_metadata.write_shape}.'
         )
 
+      # Check if the RANDOM_KEY_IMPL is the same.
+      if not isinstance(
+          array_metadata.ext_metadata, type(ref_array_metadata.ext_metadata)
+      ):
+        raise ValueError(
+            'Different ext_metadata types param:'
+            f' {param_name}. process_index={process_index} has'
+            f' {array_metadata.ext_metadata}, but'
+            f' process_index={ref_process_index} has'
+            f' {ref_array_metadata.ext_metadata}.'
+        )
+
+      if (
+          isinstance(ref_array_metadata.ext_metadata, dict)
+          and isinstance(array_metadata.ext_metadata, dict)
+      ) and (
+          (
+              ref_key := ref_array_metadata.ext_metadata.get(
+                  array_metadata_lib.RANDOM_KEY_IMPL
+              )
+          )
+          != (
+              arr_key := array_metadata.ext_metadata.get(
+                  array_metadata_lib.RANDOM_KEY_IMPL
+              )
+          )
+      ):
+        raise ValueError(
+            'ArrayMetadata Store contains different'
+            f' ext_metadata.{array_metadata_lib.RANDOM_KEY_IMPL} for param:'
+            f' {param_name}. process_index={process_index} has {ref_key}, but'
+            f' process_index={ref_process_index} has {arr_key}.'
+        )
+
 
 async def validate_all_array_metadatas(
     validator: Validator,
