@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import itertools
 import pickle
 from typing import Any, Callable
 
@@ -174,11 +173,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
         self.directory / checkpoint._LEGACY_ROOT_METADATA_FILENAME,
     )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-          [StepMetadata, RootMetadata],
-      )
+  @parameterized.product(
+      blocking_write=[True, False],
+      metadata_class=[StepMetadata, RootMetadata],
   )
   def test_write_unknown_file_path(
       self,
@@ -208,11 +205,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
           self.read_metadata_store(blocking_write).read('unknown_metadata_path')
       )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-          [StepMetadata, RootMetadata],
-      )
+  @parameterized.product(
+      blocking_write=[True, False],
+      metadata_class=[StepMetadata, RootMetadata],
   )
   def test_read_default_values(
       self,
@@ -236,11 +231,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
         metadata,
     )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-          [StepMetadata, RootMetadata],
-      )
+  @parameterized.product(
+      blocking_write=[True, False],
+      metadata_class=[StepMetadata, RootMetadata],
   )
   def test_read_with_values(
       self,
@@ -264,11 +257,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
         metadata,
     )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-          [StepMetadata, RootMetadata],
-      )
+  @parameterized.product(
+      blocking_write=[True, False],
+      metadata_class=[StepMetadata, RootMetadata],
   )
   def test_read_corrupt_json_data(
       self,
@@ -284,11 +275,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
         )
     )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-          [StepMetadata, RootMetadata],
-      )
+  @parameterized.product(
+      blocking_write=[True, False],
+      metadata_class=[StepMetadata, RootMetadata],
   )
   def test_update_without_prior_data(
       self,
@@ -297,7 +286,7 @@ class CheckpointMetadataTest(parameterized.TestCase):
   ):
     self.write_metadata_store(blocking_write).update(
         file_path=self.get_metadata_file_path(metadata_class),
-        custom={'a': 1},
+        custom_metadata={'a': 1},
     )
 
     self.write_metadata_store(blocking_write).wait_until_finished()
@@ -312,11 +301,7 @@ class CheckpointMetadataTest(parameterized.TestCase):
         ),
     )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-      )
-  )
+  @parameterized.parameters(True, False)
   def test_update_with_prior_data(
       self,
       blocking_write: bool,
@@ -329,7 +314,7 @@ class CheckpointMetadataTest(parameterized.TestCase):
 
     self.write_metadata_store(blocking_write).update(
         file_path=self.get_metadata_file_path(StepMetadata),
-        custom={'a': 1},
+        custom_metadata={'a': 1},
     )
 
     self.write_metadata_store(blocking_write).wait_until_finished()
@@ -345,11 +330,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
         ),
     )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-          [StepMetadata, RootMetadata],
-      )
+  @parameterized.product(
+      blocking_write=[True, False],
+      metadata_class=[StepMetadata, RootMetadata],
   )
   def test_update_with_unknown_kwargs(
       self,
@@ -378,11 +361,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
         ),
     )
 
-  @parameterized.parameters(
-      itertools.product(
-          [True, False],
-          [StepMetadata, RootMetadata],
-      )
+  @parameterized.product(
+      blocking_write=[True, False],
+      metadata_class=[StepMetadata, RootMetadata],
   )
   def test_write_with_read_only_store_is_no_op(
       self,
@@ -455,7 +436,7 @@ class CheckpointMetadataTest(parameterized.TestCase):
     # update validations
     self.write_metadata_store(blocking_write=False).update(
         file_path=self.get_metadata_file_path(metadata_class),
-        custom={'a': 3},
+        custom_metadata={'a': 3},
     )
     self.write_metadata_store(blocking_write=False).wait_until_finished()
     serialized_metadata = self.read_metadata_store(blocking_write=False).read(
@@ -500,8 +481,8 @@ class CheckpointMetadataTest(parameterized.TestCase):
     _ = pickle.dumps(self.read_metadata_store(blocking_write))
 
   @parameterized.parameters(
-      ({'custom': list()},),
-      ({'custom': {int(): None}},),
+      ({'custom_metadata': list()},),
+      ({'custom_metadata': {int(): None}},),
   )
   def test_deserialize_wrong_types_root_metadata(
       self, wrong_metadata: checkpoint.SerializedMetadata
@@ -512,15 +493,14 @@ class CheckpointMetadataTest(parameterized.TestCase):
   @parameterized.parameters(
       ({'item_handlers': list()},),
       ({'item_handlers': {int(): None}},),
-      ({'metrics': list()},),
+      ({'metrics': [int()]},),
       ({'metrics': {int(): None}},),
       ({'performance_metrics': list()},),
       ({'performance_metrics': {int(): float()}},),
-      ({'performance_metrics': {str(): int()}},),
       ({'init_timestamp_nsecs': float()},),
       ({'commit_timestamp_nsecs': float()},),
-      ({'custom': list()},),
-      ({'custom': {int(): None}},),
+      ({'custom_metadata': list()},),
+      ({'custom_metadata': {int(): None}},),
   )
   def test_deserialize_wrong_types_step_metadata(
       self, wrong_metadata: checkpoint.SerializedMetadata
@@ -566,7 +546,7 @@ class CheckpointMetadataTest(parameterized.TestCase):
   @parameterized.parameters(
       ('metrics', 1, dict, int),
   )
-  def test_validate_type_wrong_type(
+  def test_deserialize_metrics_kwarg(
       self, kwarg_name, kwarg_value, expected_type, wrong_type
   ):
     with self.assertRaisesRegex(
@@ -577,10 +557,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
 
   @parameterized.parameters(
       ({'metrics': 1}, dict, int),
-      ({'performance_metrics': 1}, dict, int),
       ({'init_timestamp_nsecs': 'a'}, int, str),
       ({'commit_timestamp_nsecs': 'a'}, int, str),
-      ({'custom': 1}, dict, int),
+      ({'custom_metadata': 1}, dict, int),
   )
   def test_validate_field_wrong_type(
       self, step_metadata, expected_field_type, wrong_field_type
@@ -593,6 +572,7 @@ class CheckpointMetadataTest(parameterized.TestCase):
 
   @parameterized.parameters(
       ({'item_handlers': 1}, int),
+      ({'performance_metrics': 1}, int),
   )
   def test_validate_field_sequence_wrong_type(
       self, step_metadata, wrong_field_type
@@ -609,7 +589,7 @@ class CheckpointMetadataTest(parameterized.TestCase):
       ({'item_handlers': {1: 'a'}}, str, int),
       ({'metrics': {1: 'a'}}, str, int),
       ({'performance_metrics': {1: 1.0}}, str, int),
-      ({'custom': {1: 'a'}}, str, int),
+      ({'custom_metadata': {1: 'a'}}, str, int),
   )
   def test_validate_dict_entry_wrong_key_type(
       self, step_metadata, expected_key_type, wrong_key_type
@@ -620,18 +600,6 @@ class CheckpointMetadataTest(parameterized.TestCase):
     ):
       self.deserialize_metadata(StepMetadata, step_metadata)
 
-  @parameterized.parameters(
-      ({'performance_metrics': {'a': 1}}, float, int),
-  )
-  def test_validate_dict_entry_wrong_value_type(
-      self, step_metadata, expected_value_type, wrong_value_type
-  ):
-    with self.assertRaisesRegex(
-        ValueError,
-        f'Object must be of type {expected_value_type}, got {wrong_value_type}',
-    ):
-      self.deserialize_metadata(StepMetadata, step_metadata)
-
   @parameterized.parameters(
       ({'item_handlers': {'a': 'b'}},),
       ({'performance_metrics': {'a': 1.0}},),
@@ -642,7 +610,9 @@ class CheckpointMetadataTest(parameterized.TestCase):
   ):
     expected_kwargs = kwargs.copy()
     if 'custom_metadata' in expected_kwargs:
-      expected_kwargs['custom'] = expected_kwargs.pop('custom_metadata')
+      expected_kwargs['custom_metadata'] = expected_kwargs.pop(
+          'custom_metadata'
+      )
     self.assertEqual(
         step_metadata_serialization.serialize_for_update(**kwargs),
         expected_kwargs,
