@@ -14,7 +14,6 @@
 
 """Internal IO utilities for metadata of a checkpoint at root level."""
 
-from absl import logging
 from orbax.checkpoint._src.metadata import checkpoint
 from orbax.checkpoint._src.metadata import metadata_serialization_utils as utils
 
@@ -25,34 +24,22 @@ RootMetadata = checkpoint.RootMetadata
 def serialize(metadata: RootMetadata) -> SerializedMetadata:
   """Serializes `metadata` to a dictionary."""
   return {
-      'custom': metadata.custom_metadata,
+      'custom_metadata': metadata.custom_metadata,
   }
 
 
 def deserialize(metadata_dict: SerializedMetadata) -> RootMetadata:
   """Deserializes `metadata_dict` to `RootMetadata`."""
-  validated_metadata_dict = {}
-
-  if 'custom' in metadata_dict:
-    utils.validate_type(metadata_dict['custom'], dict)
-    for k in metadata_dict.get('custom', {}) or {}:
-      utils.validate_type(k, str)
-  validated_metadata_dict['custom'] = metadata_dict.get('custom', {})
+  validated_metadata_dict = {
+      'custom_metadata': utils.validate_and_process_custom_metadata(
+          metadata_dict.get('custom_metadata', None)
+      ),
+  }
 
   for k in metadata_dict:
     if k not in validated_metadata_dict:
-      if 'custom' in metadata_dict and metadata_dict['custom']:
-        raise ValueError(
-            'Provided metadata contains unknown key %s, and the custom field '
-            'is already defined.' % k
-        )
-      logging.warning(
-          'Provided metadata contains unknown key %s. Adding it to custom.', k
+      validated_metadata_dict['custom_metadata'][k] = utils.process_unknown_key(
+          k, metadata_dict
       )
-      validated_metadata_dict['custom'][k] = metadata_dict[k]
 
-  # Rename to `custom_metadata`.
-  validated_metadata_dict['custom_metadata'] = validated_metadata_dict.pop(
-      'custom'
-  )
   return RootMetadata(**validated_metadata_dict)
