@@ -17,7 +17,7 @@
 import dataclasses
 from typing import Any
 
-from absl import logging
+from etils import epath
 from orbax.checkpoint._src.logging import step_statistics
 from orbax.checkpoint._src.metadata import checkpoint
 from orbax.checkpoint._src.metadata import metadata_serialization_utils as utils
@@ -33,6 +33,17 @@ SingleItemMetadata = checkpoint.SingleItemMetadata
 StepStatistics = step_statistics.SaveStepStatistics
 
 
+def get_step_metadata(path: epath.PathLike) -> StepMetadata:
+  """Returns StepMetadata for a given checkpoint directory."""
+  metadata_file_path = checkpoint.step_metadata_file_path(path)
+  serialized_metadata = checkpoint.metadata_store(enable_write=False).read(
+      metadata_file_path
+  )
+  if serialized_metadata is None:
+    raise ValueError(f'Step metadata not found at {metadata_file_path}')
+  return deserialize(serialized_metadata)
+
+
 def serialize(metadata: StepMetadata) -> SerializedMetadata:
   """Serializes `metadata` to a dictionary."""
 
@@ -53,8 +64,7 @@ def serialize(metadata: StepMetadata) -> SerializedMetadata:
       'performance_metrics': float_metrics,
       'init_timestamp_nsecs': metadata.init_timestamp_nsecs,
       'commit_timestamp_nsecs': metadata.commit_timestamp_nsecs,
-      # Change name from `custom_metadata` to `custom` for serialization.
-      'custom': metadata.custom_metadata,
+      'custom_metadata': metadata.custom_metadata,
   }
 
 
@@ -125,10 +135,6 @@ def serialize_for_update(**kwargs) -> SerializedMetadata:
     if k not in validated_kwargs:
       raise ValueError('Provided metadata contains unknown key %s.' % k)
 
-  # Change name from `custom_metadata` to `custom` for serialization.
-  if 'custom_metadata' in validated_kwargs:
-    validated_kwargs['custom'] = validated_kwargs.pop('custom_metadata')
-
   return validated_kwargs
 
 
@@ -140,78 +146,48 @@ def deserialize(
   """Deserializes `metadata_dict` and other kwargs to `StepMetadata`."""
   validated_metadata_dict = {}
 
-  item_handlers = metadata_dict.get('item_handlers', None)
-  if item_handlers is not None:
-    utils.validate_type(item_handlers, [dict, str])
-    if isinstance(item_handlers, CompositeCheckpointHandlerTypeStrs):
-      for k in metadata_dict.get('item_handlers', {}) or {}:
-        utils.validate_type(k, str)
-      validated_metadata_dict['item_handlers'] = item_handlers
-    elif isinstance(item_handlers, CheckpointHandlerTypeStr):
-      validated_metadata_dict['item_handlers'] = item_handlers
-  else:
-    validated_metadata_dict['item_handlers'] = None
-
-  if isinstance(item_metadata, CompositeItemMetadata):
-    validated_metadata_dict['item_metadata'] = {}
-    for k, v in item_metadata.items():
-      utils.validate_type(k, str)
-      validated_metadata_dict['item_metadata'][k] = v
-  else:
-    validated_metadata_dict['item_metadata'] = item_metadata
-
-  serialized_metrics = metadata_dict.get('metrics', None)
-  if serialized_metrics is not None:
-    utils.validate_type(serialized_metrics, dict)
-    for k in serialized_metrics:
-      utils.validate_type(k, str)
-  validated_metadata_dict['metrics'] = serialized_metrics or {}
-  if metrics is not None:
-    utils.validate_type(metrics, dict)
-    for k, v in metrics.items():
-      utils.validate_type(k, str)
-      validated_metadata_dict['metrics'][k] = v
-
-  performance_metrics = metadata_dict.get('performance_metrics', None)
-  if performance_metrics is not None:
-    utils.validate_type(performance_metrics, dict)
-    for k in performance_metrics:
-      utils.validate_type(k, str)
-      utils.validate_type(performance_metrics[k], float)
-  validated_metadata_dict['performance_metrics'] = StepStatistics(
-      **(performance_metrics or {})
+  validated_metadata_dict['item_handlers'] = (
+      utils.validate_and_process_item_handlers(
+          metadata_dict.get('item_handlers', None)
+      )
   )
 
-  init_timestamp_nsecs = metadata_dict.get('init_timestamp_nsecs', None)
-  if init_timestamp_nsecs is not None:
-    utils.validate_type(init_timestamp_nsecs, int)
-  validated_metadata_dict['init_timestamp_nsecs'] = init_timestamp_nsecs
+  validated_metadata_dict['item_metadata'] = (
+      utils.validate_and_process_item_metadata(item_metadata)
+  )
 
-  commit_timestamp_nsecs = metadata_dict.get('commit_timestamp_nsecs', None)
-  if commit_timestamp_nsecs is not None:
-    utils.validate_type(commit_timestamp_nsecs, int)
-  validated_metadata_dict['commit_timestamp_nsecs'] = commit_timestamp_nsecs
+  validated_metadata_dict['metrics'] = utils.validate_and_process_metrics(
+      metadata_dict.get('metrics', None), metrics
+  )
+
+  validated_metadata_dict['performance_metrics'] = StepStatistics(
+      **utils.validate_and_process_performance_metrics(
+          metadata_dict.get('performance_metrics', None)
+      )
+  )
+
+  validated_metadata_dict['init_timestamp_nsecs'] = (
+      utils.validate_and_process_init_timestamp_nsecs(
+          metadata_dict.get('init_timestamp_nsecs', None)
+      )
+  )
+
+  validated_metadata_dict['commit_timestamp_nsecs'] = (
+      utils.validate_and_process_commit_timestamp_nsecs(
+          metadata_dict.get('commit_timestamp_nsecs', None)
+      )
+  )
+
+  validated_metadata_dict['custom_metadata'] = (
+      utils.validate_and_process_custom_metadata(
+          metadata_dict.get('custom_metadata', None)
+      )
+  )
 
-  custom = metadata_dict.get('custom', None)
-  if custom is not None:
-    utils.validate_type(custom, dict)
-    for k in custom:
-      utils.validate_type(k, str)
-  validated_metadata_dict['custom'] = custom or {}
   for k in metadata_dict:
     if k not in validated_metadata_dict:
-      if 'custom' in metadata_dict and metadata_dict['custom']:
-        raise ValueError(
-            'Provided metadata contains unknown key %s, and the custom field '
-            'is already defined.' % k
-        )
-      logging.warning(
-          'Provided metadata contains unknown key %s. Adding it to custom.', k
+      validated_metadata_dict['custom_metadata'][k] = utils.process_unknown_key(
+          k, metadata_dict
       )
-      validated_metadata_dict['custom'][k] = metadata_dict[k]
-  # Change name from `custom` to `custom_metadata` for deserialization.
-  validated_metadata_dict['custom_metadata'] = validated_metadata_dict.pop(
-      'custom'
-  )
 
   return StepMetadata(**validated_metadata_dict)
