@@ -38,6 +38,7 @@ import numpy as np
 from orbax.checkpoint._src.arrays import subchunking
 from orbax.checkpoint._src.arrays import types as arrays_types
 from orbax.checkpoint._src.futures import future
+from orbax.checkpoint._src.metadata import array_metadata as array_metadata_lib
 from orbax.checkpoint._src.metadata import array_metadata_store as array_metadata_store_lib
 from orbax.checkpoint._src.metadata import empty_values
 from orbax.checkpoint._src.metadata import sharding as sharding_metadata
@@ -58,6 +59,7 @@ RestoreArgs: TypeAlias = types.RestoreArgs
 TypeHandler: TypeAlias = types.TypeHandler
 TypeHandlerRegistry: TypeAlias = types.TypeHandlerRegistry
 Pytree = Any
+PLACEHOLDER = ...
 
 Layout = layout.Layout
 Shape = arrays_types.Shape
@@ -213,6 +215,7 @@ def _build_array_write_spec(
     use_ocdbt: bool,
     process_index: Optional[Union[int, str]] = None,
     metadata_key: Optional[str] = None,
+    ext_metadata: Optional[Dict[str, Any]] = None,
 ) -> ts_utils.ArrayWriteSpec:
   """Gets ArrayWriteSpec for writing."""
   if info.path is None:
@@ -235,6 +238,7 @@ def _build_array_write_spec(
       process_id=process_index,
       ocdbt_target_data_file_size=info.ocdbt_target_data_file_size,
       metadata_key=metadata_key,
+      ext_metadata=ext_metadata,
   )
 
 
@@ -249,10 +253,16 @@ def check_input_arguments(*args):
       raise ValueError('Found input args with mismatched lengths.')
 
 
-def check_array_values(values: Sequence[Union[jax.Array, np.ndarray]]):
-  for v in values:
+def check_array_values(
+    values: Sequence[Union[jax.Array, np.ndarray]],
+    infos: Sequence[types.ParamInfo],
+):
+  for v, info in zip(values, infos):
     if v.size == 0:
-      raise ValueError('Cannot save arrays with zero size.')
+      raise ValueError(
+          f'Cannot save arrays with zero size: ParamInfo: [name={info.name},'
+          f'value_typestr={info.value_typestr}]'
+      )
 
 
 async def _validate_params(
@@ -637,7 +647,7 @@ class NumpyHandler(types.TypeHandler):
     """Uses Tensorstore to serialize a numpy array."""
     args = args or [types.SaveArgs()] * len(values)
     check_input_arguments(values, infos, args)
-    check_array_values(values)
+    check_array_values(values, infos)
     if logging.vlog_is_on(1):
       _print_ts_debug_data(self._metadata_key, infos)
     copied_values = [copy.deepcopy(v) for v in values]
@@ -734,7 +744,14 @@ class ScalarHandler(NumpyHandler):
     for r in results:
       if r.ndim != 0:
         raise ValueError('Restored result is not a scalar.')
-    return [r.item() for r in results]
+    results = [r.item() for r in results]
+    if args:
+      # Cast to the intended `restore_type` if it is provided.
+      return [
+          a.restore_type(r) if a.restore_type else r
+          for a, r in zip(args, results)
+      ]
+    return results
 
   def memory_size(self, values: Sequence[Scalar]) -> Sequence[Tuple[int, int]]:  # pytype: disable=signature-mismatch
     actual_sizes = [sys.getsizeof(v) for v in values]
@@ -752,12 +769,11 @@ def get_sharding_tensorstore_spec(
   kvstore_tspec = ts_utils.build_kvstore_tspec(
       directory, name=_SHARDING, use_ocdbt=False
   )
+  param_name = base64.urlsafe_b64encode(param_name.encode()).decode('utf-8')
   return {
       'driver': 'json',
       'kvstore': kvstore_tspec,
-      'json_pointer': '/' + base64.urlsafe_b64encode(
-          param_name.encode()
-      ).decode('utf-8'),
+      'json_pointer': f'/{param_name}',
   }
 
 
@@ -863,7 +879,7 @@ class ArrayHandler(types.TypeHandler):
       metadata_key: Optional[str] = None,
       primary_host: Optional[int] = 0,
       replica_id: Optional[int] = 0,
-      use_replica_parallel: bool = False,
+      use_replica_parallel: bool = True,
       enable_write_sharding_file: bool = True,
       array_metadata_store: array_metadata_store_lib.Store | None = None,
   ):
@@ -889,6 +905,7 @@ class ArrayHandler(types.TypeHandler):
     self._enable_write_sharding_file = enable_write_sharding_file
     self._use_replica_parallel = use_replica_parallel
     self._array_metadata_store = array_metadata_store
+    self._ext_metadata = dict()
 
     logging.vlog(
         1,
@@ -925,6 +942,7 @@ class ArrayHandler(types.TypeHandler):
         use_ocdbt=use_ocdbt,
         process_index=process_index,
         metadata_key=self._metadata_key,
+        ext_metadata=self._ext_metadata.get(info.name),
     )
 
   def _get_json_tspec_read(
@@ -1082,6 +1100,7 @@ class ArrayHandler(types.TypeHandler):
               process_index=multihost.process_index(),
           )
       )
+
     await asyncio.gather(*write_coros)
     await sharding_metadata_txn.commit_async()
     if ocdbt_transaction is not None:
@@ -1094,22 +1113,43 @@ class ArrayHandler(types.TypeHandler):
       args: Optional[Sequence[types.SaveArgs]] = None,
   ) -> Sequence[future.Future]:
     """See superclass documentation."""
-    for v in values:
+    args = args or [types.SaveArgs()] * len(values)
+    check_input_arguments(values, infos, args)
+    check_array_values(values, infos)
+
+    self._ext_metadata = dict()
+    arrays = []
+    for v, info in zip(values, infos):
       if (
           isinstance(v, jax.Array)
           and jax.process_count() > 1
           and v.is_fully_addressable
       ):
+        debug_param_info = (
+            f'ParamInfo=[name={info.name},value_typestr={info.value_typestr}]'
+        )
+        debug_array = (
+            f'jax.Array=[value={v},shape={v.shape},dtype={v.dtype},'
+            f'sharding={v.sharding},device={v.device}]'
+        )
         raise ValueError(
-            'Cannot serialize host local arrays. Arrays like this are typically'
-            ' obtained using pmap. Consider using'
+            f'Cannot serialize host local jax.Array ({debug_param_info},'
+            f' {debug_array}) in multi-host setting. Arrays like this are'
+            ' typically obtained using pmap. Consider using'
             ' fully_replicated_host_local_array_to_global_array in'
             ' orbax/checkpoint/utils.py to convert your arrays into'
-            ' serializable objects.'
+            f' serializable objects. Array.sharding: {v.sharding}'
         )
-    args = args or [types.SaveArgs()] * len(values)
-    check_input_arguments(values, infos, args)
-    check_array_values(values)
+
+      if jax.dtypes.issubdtype(v.dtype, jax.dtypes.prng_key):
+        # a JAX random key
+        arrays.append(jax.random.key_data(v))
+        self._ext_metadata[info.name] = {
+            array_metadata_lib.RANDOM_KEY_IMPL: str(jax.random.key_impl(v))
+        }
+      else:
+        # regular array
+        arrays.append(v)
 
     assert all([info.enable_pinned_host_transfer for info in infos]) or all(
         [not info.enable_pinned_host_transfer for info in infos]
@@ -1117,7 +1157,7 @@ class ArrayHandler(types.TypeHandler):
 
     # Complete D2H transfer in parallel for each array.
     values_on_host = replica_slices.transfer_arrays_to_host(
-        values,
+        arrays,
         self._replica_id,
         self._use_replica_parallel,
         enable_pinned_host_transfer=infos[0].enable_pinned_host_transfer,
@@ -1130,6 +1170,41 @@ class ArrayHandler(types.TypeHandler):
         )
     ]
 
+  def _parse_array_metadatas(self, array_metadatas, infos, deserialized_arrays):
+    """Parse array_metadatas and adjust deserialized_arrays accordingly."""
+
+    logging.vlog(1, 'array_metadatas = %s', array_metadatas)
+    if not isinstance(array_metadatas, Dict):
+      raise ValueError(
+          'Expecting array_metadatas to be a "Dict" but got'
+          f' {type(array_metadatas)}.'
+      )
+
+    # use the first available array_metadata
+    array_metadatas_cache = {
+        array_metadata.param_name: array_metadata
+        for array_metadata in next(iter(array_metadatas.values()))
+    }
+
+    for i, (info, v) in enumerate(zip(infos, deserialized_arrays)):
+      if meta := array_metadatas_cache.get(info.name):
+        assert isinstance(
+            meta, array_metadata_lib.SerializedArrayMetadata
+        ), f'Expecting SerializedArrayMetadata but got {type(meta)}.'
+        if meta.ext_metadata is None or not isinstance(meta.ext_metadata, dict):
+          continue
+
+        if impl := meta.ext_metadata.get(array_metadata_lib.RANDOM_KEY_IMPL):
+          deserialized_arrays[i] = jax.random.wrap_key_data(v, impl=impl)
+          logging.vlog(
+              1,
+              '%s: recreated as a random key: %s',
+              info.name,
+              deserialized_arrays[i],
+          )
+
+    return deserialized_arrays
+
   async def deserialize(
       self,
       infos: Sequence[types.ParamInfo],
@@ -1229,21 +1304,33 @@ class ArrayHandler(types.TypeHandler):
               strict=arg.strict if hasattr(arg, 'strict') else True,
           )
       ]
-    ret = await asyncio.gather(*deserialize_ops)
+
+    if self._array_metadata_store is not None:
+      deserialize_ops.append(
+          self._array_metadata_store.read(
+              checkpoint_dir=infos[0].parent_dir,
+          )
+      )
+      *ret, array_metadatas = await asyncio.gather(*deserialize_ops)
+
+      if array_metadatas:
+        ret = self._parse_array_metadatas(array_metadatas, infos, ret)
+    else:
+      ret = await asyncio.gather(*deserialize_ops)
 
     if logging.vlog_is_on(1):
       for a in ret:
         logging.vlog(
             1,
             'restored jax.Array.shape = %s, jax.array.dtype = %s,'
-            ' jax.array.layout + %s',
-            a.shape,
-            a.dtype,
-            a.layout,
+            ' jax.array.layout = %s',
+            getattr(a, 'shape', None),
+            getattr(a, 'dtype', None),
+            getattr(a, 'layout', None),
         )
       _print_ts_debug_data(self._metadata_key, infos)
 
-    return ret
+    return ret  # pytype: disable=bad-return-type
 
   def memory_size(
       self, values: Sequence[jax.Array]
@@ -1607,6 +1694,34 @@ class StringHandler(types.TypeHandler):
     return list(zip(write_sizes, read_sizes))
 
 
+class PlaceholderHandler(types.TypeHandler):
+  """TypeHandler for placeholders."""
+
+  def typestr(self) -> str:
+    return 'placeholder'
+
+  async def metadata(self, infos: Sequence[types.ParamInfo]) -> Sequence[Any]:
+    raise NotImplementedError('Placeholders do not have metadata.')
+
+  async def serialize(
+      self,
+      values: Sequence[str],
+      infos: Sequence[types.ParamInfo],
+      args: Sequence[types.SaveArgs] | None = None,
+  ) -> Sequence[future.Future]:
+    """See superclass documentation."""
+    raise NotImplementedError('Placeholders cannot be serialized.')
+
+  async def deserialize(
+      self,
+      infos: Sequence[ParamInfo],
+      args: Sequence[RestoreArgs] | None = None,
+  ) -> Sequence[Any]:
+    """See superclass documentation."""
+    del args
+    return [PLACEHOLDER] * len(infos)
+
+
 class _TypeHandlerRegistryImpl(types.TypeHandlerRegistry):
   """The implementation for TypeHandlerRegistry."""
 
@@ -1710,8 +1825,12 @@ _DEFAULT_TYPE_HANDLERS = tuple([
     (bytes, ScalarHandler()),
     (np.number, ScalarHandler()),
     (np.ndarray, NumpyHandler()),
-    (jax.Array, ArrayHandler()),
+    (
+        jax.Array,
+        ArrayHandler(array_metadata_store=array_metadata_store_lib.Store()),
+    ),
     (str, StringHandler()),
+    (type(PLACEHOLDER), PlaceholderHandler()),
 ])
 
 
