@@ -22,7 +22,7 @@ import datetime
 import threading
 import time
 import typing
-from typing import Any, Callable, Container, Iterable, List, Mapping, Optional, Sequence, Tuple, Type, Union
+from typing import Any, Callable, Container, Iterable, List, Mapping, Optional, Sequence, Tuple, Type, Union, overload
 
 from absl import logging
 from etils import epath
@@ -34,6 +34,7 @@ from orbax.checkpoint import args as args_lib
 from orbax.checkpoint import checkpoint_args
 from orbax.checkpoint import options as options_lib
 from orbax.checkpoint import utils
+from orbax.checkpoint._src import threading as threading_lib
 from orbax.checkpoint._src.checkpoint_managers import save_decision_policy as save_decision_policy_lib
 from orbax.checkpoint._src.checkpointers import abstract_checkpointer
 from orbax.checkpoint._src.checkpointers import async_checkpointer
@@ -43,17 +44,18 @@ from orbax.checkpoint._src.handlers import composite_checkpoint_handler
 from orbax.checkpoint._src.handlers import handler_registration
 from orbax.checkpoint._src.handlers import json_checkpoint_handler
 from orbax.checkpoint._src.handlers import proto_checkpoint_handler
+from orbax.checkpoint._src.logging import abstract_logger
+from orbax.checkpoint._src.logging import standard_logger
+from orbax.checkpoint._src.logging import step_statistics
 from orbax.checkpoint._src.metadata import checkpoint
 from orbax.checkpoint._src.metadata import checkpoint_info
 from orbax.checkpoint._src.metadata import root_metadata_serialization
+from orbax.checkpoint._src.metadata import step_metadata_serialization
 from orbax.checkpoint._src.multihost import multihost
 from orbax.checkpoint._src.path import atomicity_types
 from orbax.checkpoint._src.path import deleter
 from orbax.checkpoint._src.path import step as step_lib
 from orbax.checkpoint._src.path import utils as path_utils
-from orbax.checkpoint.logging import abstract_logger
-from orbax.checkpoint.logging import standard_logger
-from orbax.checkpoint.logging import step_statistics
 from typing_extensions import Self  # for Python version < 3.11
 
 
@@ -68,6 +70,9 @@ CheckpointersDict = Mapping[str, AbstractCheckpointer]
 AbstractCheckpointManager = (
     abstract_checkpoint_manager.AbstractCheckpointManager
 )
+StepMetadata = checkpoint.StepMetadata
+RootMetadata = checkpoint.RootMetadata
+ItemMetadata = checkpoint.CompositeItemMetadata | checkpoint.SingleItemMetadata
 AsyncCheckpointer = async_checkpointer.AsyncCheckpointer
 Checkpointer = checkpointer_lib.Checkpointer
 JsonCheckpointHandler = json_checkpoint_handler.JsonCheckpointHandler
@@ -103,14 +108,6 @@ def _metrics_file_exists(metrics_item_path: epath.Path) -> bool:
   )
 
 
-def _descriptor_file_exists(descriptor_item_path: epath.Path) -> bool:
-  """True if item directory AND actual file both exist."""
-  return (
-      descriptor_item_path.exists()
-      and (descriptor_item_path / f'{DESCRIPTOR_ITEM_NAME}.pbtxt').exists()
-  )
-
-
 class StepAlreadyExistsError(ValueError):
   """Raised when a step is already present for a save request."""
 
@@ -125,7 +122,6 @@ class _FinalizeThread(threading.Thread):
   def __init__(
       self,
       step: int,
-      save_thread: threading.Thread,
       target: Callable[..., object],
       name: str,
       args=(),
@@ -141,14 +137,11 @@ class _FinalizeThread(threading.Thread):
         daemon=daemon,
     )
     self._step = step
-    self._save_thread = save_thread
+    self._thread_local = threading.local()
 
   def step(self) -> int:
     return self._step
 
-  def save_thread(self) -> threading.Thread:
-    return self._save_thread
-
   def run(self):
     try:
       super().run()
@@ -157,7 +150,13 @@ class _FinalizeThread(threading.Thread):
 
   def join(self, *args, **kwargs):
     super().join(*args, **kwargs)
-    if self.exception:
+
+    # Keep track of whether the calling thread should raise the exception.
+    if not hasattr(self._thread_local, 'should_raise'):
+      self._thread_local.should_raise = True
+    if self.exception and self._thread_local.should_raise:
+      # Each thread should only raise once.
+      self._thread_local.should_raise = False
       raise self.exception
 
 
@@ -169,8 +168,8 @@ class _ShouldSaveFnPolicy(save_decision_policy_lib.SaveDecisionPolicy):
 
   def should_save(
       self,
-      step: save_decision_policy_lib.StepInfo,
-      previous_steps: Sequence[save_decision_policy_lib.StepInfo],
+      step: checkpoint_info.CheckpointInfo,
+      previous_steps: Sequence[checkpoint_info.CheckpointInfo],
       *,
       context: save_decision_policy_lib.DecisionContext,
   ) -> bool:
@@ -662,6 +661,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         enable_write=True, blocking_write=True
     )
 
+    self._default_item = threading_lib.OptionalRef[bool]()
     if checkpointers is not None:
       jax.monitoring.record_event(
           '/jax/orbax/deprecation/checkpoint_manager_legacy_init'
@@ -672,7 +672,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
           ' https://orbax.readthedocs.io/en/latest/api_refactor.html to'
           ' migrate.'
       )
-      self._default_item = isinstance(checkpointers, AbstractCheckpointer)
+      self._default_item.set(isinstance(checkpointers, AbstractCheckpointer))
       self._checkpointer = self._configure_checkpointer_legacy_init(
           checkpointers, self._options
       )
@@ -681,7 +681,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
       # not, detemine this lazily instead on the first call to `save`, `restore`
       # or `item_metadata`. Once locked-in, the value of `_default_item` will
       # not change.
-      self._default_item = None
+      self._default_item.set(None)
       self._checkpointer = self._configure_checkpointer_from_handler_registry(
           handler_registry,
           self._options,
@@ -690,20 +690,21 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
       # In this case, we can just default construct the
       # CheckpointHandlerRegistry and allow the user to lazily specify default
       # vs. multi-item mode.
-      self._default_item = None
+      self._default_item.set(None)
       handler_registry = handler_registration.DefaultCheckpointHandlerRegistry()
       self._checkpointer = self._configure_checkpointer_from_handler_registry(
           handler_registry,
           self._options,
       )
     else:
-      self._default_item = isinstance(item_handlers, CheckpointHandler)
+      default_item = isinstance(item_handlers, CheckpointHandler)
+      self._default_item.set(default_item)
       self._checkpointer = (
           self._configure_checkpointer_from_item_names_and_handlers(
               item_names,
               item_handlers,
               self._options,
-              self._default_item,
+              default_item,
           )
       )
 
@@ -747,20 +748,19 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
 
     self._metadata_dir = self.directory / METADATA_ITEM_NAME
     if self._options.read_only and not self._metadata_dir.exists():
-      self._metadata = {} if metadata is None else metadata
+      custom_metadata = {} if metadata is None else dict(metadata)
     else:
-      self._metadata = None
+      custom_metadata = None
+    self._root_metadata = threading_lib.Ref(
+        RootMetadata(
+            custom_metadata=custom_metadata,
+        )
+    )
 
-    self._maybe_save_metadata(metadata)
+    self._maybe_save_root_metadata(metadata)
 
     # TODO: b/359854428 - Move Finalize biz logic to a separate class/module.
-    self._finalize_thread_lock = threading.Lock()
-    with self._finalize_thread_lock:
-      self._finalize_thread = None
-
-    self._is_saving_in_progress_lock = threading.Lock()
-    with self._is_saving_in_progress_lock:
-      self._is_saving_in_progress = False
+    self._finalize_thread = threading_lib.OptionalRef[_FinalizeThread]()
 
     self._checkpoint_deleter: deleter.CheckpointDeleter = (
         deleter.create_checkpoint_deleter(
@@ -993,11 +993,19 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
 
   @property
   def directory(self) -> epath.Path:
-    """See superclass documentation."""
+    """Returns the top-level directory containing checkpoints for all items."""
     return self._directory
 
   def all_steps(self, read: bool = False) -> Sequence[int]:
-    """See superclass documentation."""
+    """Returns all steps tracked by the manager.
+
+    Args:
+      read: If True, forces a read directly from the storage location.
+        Otherwise, a cached result can be returned.
+
+    Returns:
+      A sequence of steps (integers)
+    """
     if read:
       logging.warning(
           '`read` option is deprecated. Use `reload` to read from disk.'
@@ -1006,12 +1014,24 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
     return [ckpt.step for ckpt in self._checkpoints]
 
   def latest_step(self) -> Optional[int]:
-    """See superclass documentation."""
+    """Returns the latest step saved.
+
+    Returns None if no steps have been saved.
+
+    Returns:
+      A step (int) or None if no steps are present.
+    """
     latest = self._checkpoints.latest()
     return latest.step if latest else None
 
   def best_step(self) -> Optional[int]:
-    """See superclass documentation."""
+    """Returns the best step saved, as defined by `options.best_fn`.
+
+    Returns None if no steps have been saved.
+
+    Returns:
+      A step (int) or None if no steps are present.
+    """
     if not self._track_best:
       return self.latest_step()
     if self._checkpoints.empty():
@@ -1030,11 +1050,20 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
     self._checkpoints.set(self._load_checkpoint_infos())
 
   def reached_preemption(self, step: int) -> bool:
-    """See superclass documentation."""
+    """Returns True if a preemption sync point has been reached."""
     return utils.reached_preemption(step)
 
   def should_save(self, step: int) -> bool:
-    """See superclass documentation."""
+    """Returns True if a checkpoint should be saved for the current step.
+
+    This depends the previous step and SaveDecisionPolicy.
+
+    Args:
+      step: int
+
+    Returns:
+      True if the checkpoint should be saved.
+    """
     if self._options.read_only:
       logging.warning('%s is read only, save will be skipped', self.directory)
       return False
@@ -1046,17 +1075,16 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
 
     is_saving_in_progress = self.is_saving_in_progress()
     reached_preemption = self.reached_preemption(step)
-    previous_step_infos = [
-        save_decision_policy_lib.StepInfo(step=ckpt.step)
-        for ckpt in self._checkpoints
-    ]
-    current_step_info = save_decision_policy_lib.StepInfo(step=step)
+    current_step_info = checkpoint_info.CheckpointInfo(
+        step=step, time=datetime.datetime.now(tz=datetime.timezone.utc),
+        metrics=None
+    )
     context = save_decision_policy_lib.DecisionContext(
         is_saving_in_progress=is_saving_in_progress,
         reached_preemption=reached_preemption,
     )
     return self._save_decision_policy.should_save(
-        current_step_info, previous_steps=previous_step_infos, context=context
+        current_step_info, previous_steps=self._checkpoints, context=context
     )
 
   def _get_save_directory(
@@ -1123,7 +1151,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         raise ValueError(
             f'Expected args of type `CheckpointArgs`; found {type(args)}.'
         )
-      if self._default_item:
+      if self._default_item.get():
         if isinstance(args, args_lib.Composite):
           raise ValueError(
               'Cannot provide `args` of type `Composite` when dealing with a'
@@ -1146,17 +1174,88 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
       args: Optional[args_lib.CheckpointArgs] = None,
       custom_metadata: dict[str, Any] | None = None,
   ) -> bool:
-    """See superclass documentation."""
+    """Saves the provided items.
+
+    This method should be called by all hosts - process synchronization and
+    actions that need to be performed on only one host are managed internally.
+
+    NOTE: The `items` and `save_kwargs` arguments are deprecated, use `args`
+    instead. Make sure to configure `CheckpointManager` with `item_names`.
+
+    `args` should be a subclass of
+    `orbax.checkpoint.args.CheckpointArgs`, the specific type of which is used
+    to indicate what logic is used to save the object. For a typical, PyTree of
+    arrays, use `StandardSave`/`StandardRestore`.
+
+    When constructing the `CheckpointManager`, if no `item_names` were provided,
+    it is assumed that we are managing a single object. If `item_names` were
+    provided, it is assumed that we are managing multiple objects, and `args`
+    must be `orbax.checkpoint.args.CompositeArgs`. See below for details.
+
+    Example::
+
+      # Single item
+      mngr = ocp.CheckpointManager(directory)
+      mngr.save(step, args=ocp.args.StandardSave(my_train_state))
+
+      # Multiple items
+      mngr = ocp.CheckpointManager(directory, item_names=('state', 'meta'))
+      mngr.save(step, args=ocp.args.Composite(
+          state=ocp.args.StandardSave(my_train_state),
+          meta=ocp.args.JsonSave(my_metadata)
+      ))
+
+    Args:
+      step: current step, int
+      items: a savable object, or a dictionary of object name to savable object.
+      save_kwargs: save kwargs for a single Checkpointer, or a dictionary of
+        object name to kwargs needed by the Checkpointer implementation to save
+        the object.
+      metrics: a dictionary of metric name (string) to numeric value to be
+        tracked along with this checkpoint. Required if `options.best_fn` is
+        set. Allows users to specify a metric value to determine which
+        checkpoints are best and should be kept (in conjunction with
+        `options.max_to_keep`).
+      force: if `True`, this method will attempt to save a checkpoint regardless
+        of the result of `AbstractCheckpointManager.should_save(step)`. By
+        default, `save` will only write a checkpoint to disk when the options
+        permit, e.g. when `step` is in `options.save_interval_steps` or
+        `options.save_on_steps`. Setting `force=True` will not overwrite
+        existing checkpoints.
+      args: `CheckpointArgs` which is used to save checkpointable objects with
+        the appropriate logic.
+      custom_metadata: a dictionary of custom metadata to be written to the
+        checkpoint directory via StepMetadata.
+
+    Returns:
+      bool indicating whether a save operation was performed.
+    Raises:
+      ValueError: if `track_best` was indicated but `metrics` is not provided.
+      ValueError: directory creation failed.
+      ValueError: if an item is provided for which no `Checkpointer` is
+      found.
+      ValueError: if the checkpoint already exists.
+    """
     process_index = multihost.process_index()
     step_stats = step_statistics.SaveStepStatistics()
     step_stats.step = step
     step_stats.checkpoint_manager_blocking_start_time = time.time()
     step_stats.directory = str(self.directory)
 
+    multihost.sync_global_processes(
+        multihost.unique_barrier_key(
+            'CheckpointManager:save_start',
+            prefix=self._multiprocessing_options.barrier_sync_key_prefix,
+        ),
+        processes=self._multiprocessing_options.active_processes,
+        record_event_name=(
+            '/jax/orbax/write/checkpoint_start_sync_duration_secs'
+        ),
+    )
+
     if items is None and args is None:
       raise ValueError('Must provide `args` for `save`.')
-    if self._default_item is None:
-      self._default_item = determine_default_item_mode_from_args(args)
+    self._default_item.set_if_none(determine_default_item_mode_from_args(args))
     self._validate_args(items, args)
     if not force and not self.should_save(step):
       return False
@@ -1192,7 +1291,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
       items = {}
     if save_kwargs is None:
       save_kwargs = {}
-    if self._default_item:
+    if self._default_item.get():
       items = {DEFAULT_ITEM_NAME: items}
       save_kwargs = {DEFAULT_ITEM_NAME: save_kwargs}
 
@@ -1211,7 +1310,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         args_dict[key] = save_ckpt_arg_cls(item, **extra_args)  # pylint: disable=too-many-function-args  # pytype: disable=wrong-arg-count
       args = args_lib.Composite(**args_dict)
     else:
-      if self._default_item:
+      if self._default_item.get():
         args = args_lib.Composite(**{DEFAULT_ITEM_NAME: args})
       else:
         if not isinstance(args, args_lib.Composite):
@@ -1300,34 +1399,40 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         processes=self._multiprocessing_options.active_processes,
     )
 
-    assert self._finalize_thread is None
+    current_thread = threading.current_thread()
     if is_async_checkpointer(self._checkpointer):
-      with self._is_saving_in_progress_lock:
-        self._is_saving_in_progress = True
-      with self._finalize_thread_lock:
+
+      def launch_finalize_thread() -> _FinalizeThread:
+        assert not self.is_saving_in_progress(), (
+            'Save finalization already in progress for'
+            f' step={self._finalize_thread.get_not_none().step()}'
+        )
         finalize_thread_name = 'save_finalize'
         logging.info(
             '[process=%s][thread=%s][step=%s] Starting CheckpointManager Save'
             ' Finalize thread=%s',
             process_index,
-            threading.current_thread().name,
+            current_thread.name,
             step,
             finalize_thread_name,
         )
-        self._finalize_thread = _FinalizeThread(
+        finalize_thread = _FinalizeThread(
             step=step,
-            save_thread=threading.current_thread(),
             name=finalize_thread_name,
             target=self._finalize,
             args=(step, steps_to_remove),
         )
-        self._finalize_thread.start()
+        finalize_thread.start()
+        return finalize_thread
+
+      self._finalize_thread.set_from(launch_finalize_thread)
+
     else:
       self._finalize(step, steps_to_remove)
       logging.info(
           '[process=%s][thread=%s][step=%s] Finished synchronous save.',
           process_index,
-          threading.current_thread().name,
+          current_thread.name,
           step,
       )
 
@@ -1338,8 +1443,10 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
     self._logger.log_entry(dataclasses.asdict(step_stats))
     return True
 
-  def _maybe_get_default_item(self, composite_result: args_lib.Composite):
-    if self._default_item:
+  def _maybe_get_default_item(
+      self, composite_result: args_lib.Composite
+  ) -> Union[Any, args_lib.Composite]:
+    if self._default_item.get():
       if DEFAULT_ITEM_NAME not in composite_result:
         raise ValueError(
             'Unable to retrieve default item. Please ensure that a handler for'
@@ -1359,7 +1466,55 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
       directory: Optional[epath.PathLike] = None,
       args: Optional[args_lib.CheckpointArgs] = None,
   ) -> Union[Any, Mapping[str, Any]]:
-    """See superclass documentation."""
+    """Restores from the given step and provided items.
+
+    This method should be called by all hosts - process synchronization and
+    actions that need to be performed on only one host are managed internally.
+
+    NOTE: The `items` and `restore_kwargs` arguments are deprecated, use `args`
+    instead. Make sure to configure `CheckpointManager` with `item_names`.
+    See `save` docstring for additional details.
+
+    Example::
+
+      # Single item
+      mngr = ocp.CheckpointManager(directory)
+      mngr.restore(step, args=ocp.args.StandardRestore(abstract_train_state))
+
+      # Multiple items
+      mngr = ocp.CheckpointManager(directory, item_names=('state', 'meta'))
+      mngr.restore(step, args=ocp.args.Composite(
+          state=ocp.args.StandardRestore(abstract_train_state),
+          meta=ocp.args.JsonRestore(),
+      ))
+      # If it is acceptable to restore without providing additional arguments,
+      # and if a save has already been performed, it is ok to do the following:
+      mngr.restore(step, args=ocp.args.Composite(state=None, meta=None))
+      # If a save has not already been performed, there is no way for Orbax to
+      # know how to restore the objects. If a save has already been performed,
+      # it remembers the logic used to save the objects.
+
+    Args:
+      step: current step, int
+      items: a restoreable object, or a dictionary of object name to restorable
+        object.
+      restore_kwargs: restore kwargs for a single Checkpointer, or a dictionary
+        of object name to kwargs needed by the Checkpointer implementation to
+        restore the object.
+      directory: if provided, uses the given directory rather than the
+        `directory` property of this class. Can be used to restore checkpoints
+        from an independent location.
+      args: `CheckpointArgs` which is used to restore checkpointable objects
+        with the appropriate logic.
+
+    Returns:
+      If managing a single item, returns a single checkpointable object.
+      If managing multiple items, returns ocp.args.Composite, where the keys
+      are item names, and values are checkpointable objects.
+
+    Raises:
+      FileNotFoundError: If no steps are found in the directory.
+    """
     if step is None:
       step = self.latest_step()
       if step is None:
@@ -1371,19 +1526,20 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
     step_stats.checkpoint_manager_start_time = time.time()
     step_stats.directory = str(directory)
 
-    if self._default_item is None:
-      self._default_item = _determine_default_item_mode_from_directory(
-          self._get_read_step_directory(step, directory)
-      )
+    self._default_item.set_if_none(
+        _determine_default_item_mode_from_directory(
+            self._get_read_step_directory(step, directory)
+        )
+    )
     self._validate_args(items, args)
 
     if items is None:
       items = {}
-    elif self._default_item:
+    elif self._default_item.get():
       items = {DEFAULT_ITEM_NAME: items}
     if restore_kwargs is None:
       restore_kwargs = {}
-    elif self._default_item:
+    elif self._default_item.get():
       restore_kwargs = {DEFAULT_ITEM_NAME: restore_kwargs}
 
     if args is None:
@@ -1400,7 +1556,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         args_dict[key] = restore_ckpt_arg_cls(item, **extra_args)  # pylint: disable=too-many-function-args  # pytype: disable=wrong-arg-count
       args = args_lib.Composite(**args_dict)
     else:
-      if self._default_item:
+      if self._default_item.get():
         args = args_lib.Composite(**{DEFAULT_ITEM_NAME: args})
       else:
         args = typing.cast(args_lib.Composite, args)
@@ -1419,9 +1575,14 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
 
     return self._maybe_get_default_item(restored)
 
-  def item_metadata(self, step: int) -> Union[Any, args_lib.Composite]:
+  def item_metadata(
+      self, step: int
+  ) -> Union[Any, args_lib.Composite, ItemMetadata]:
     """Retrieves metadata for all known items.
 
+    Important note: This method will soon be deprecated in favor of
+    `metadata().item_metadata`. Please use that method instead.
+
     Note that metadata will only be returned for items that can actually be
     interpreted. If an item is present in the checkpoint but not registered
     (using a prior save or restore, or with `handler_registry` at init), the
@@ -1434,18 +1595,9 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
       Either metadata for the item itself, if in default-item mode, or a
       Composite of metadata for each item.
     """
-    assert isinstance(self._checkpointer.handler, CompositeCheckpointHandler)
-    read_step_directory = self._get_read_step_directory(step, self.directory)
-
-    result = self._checkpointer.metadata(read_step_directory)
-    if isinstance(result, checkpoint.StepMetadata):
-      result = result.item_metadata
-    if self._default_item is None:
-      self._default_item = _determine_default_item_mode_from_directory(
-          read_step_directory
-      )
-    return self._maybe_get_default_item(result)
+    return self.metadata(step).item_metadata
 
+  # TODO(b/370812224): Deprecate in favor of StepMetadata.metrics
   def metrics(self, step: int) -> Optional[PyTree]:
     if self._track_best:
       try:
@@ -1536,61 +1688,111 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         )
     )
 
-  def _metadata_file_path(self, legacy: bool = False) -> epath.Path:
+  def _root_metadata_file_path(self, legacy: bool = False) -> epath.Path:
     if not self._metadata_dir.exists():
-      raise ValueError('Metadata directory is not initialized.')
+      raise ValueError(
+          f'Root Metadata directory is not initialized: {self._metadata_dir}'
+      )
     return checkpoint.root_metadata_file_path(self._metadata_dir, legacy=legacy)
 
-  def _maybe_save_metadata(self, metadata: Mapping[str, Any]):
+  def _maybe_save_root_metadata(
+      self, custom_metadata: Mapping[str, Any] | None
+  ):
     """Saves CheckpointManager level metadata, skips if already present."""
     if self._options.save_root_metadata:
-      logging.info('Saving root metadata')
       if (
-          metadata is not None
+          custom_metadata is not None
           and not self._options.read_only
           and utils.is_primary_host(self._multiprocessing_options.primary_host)
       ):
-        logging.info('Creating metadata directory')
         self._metadata_dir.mkdir(parents=True, exist_ok=True)
-        file_path = self._metadata_file_path()
+        file_path = self._root_metadata_file_path()
         if not file_path.exists():  # May have been created by a previous run.
-          logging.info('Writing root metadata')
-          metadata_to_save = checkpoint.RootMetadata(
-              custom_metadata=dict(metadata),
-          )
+          metadata_to_save = self._root_metadata.get()
+          if custom_metadata is not None:
+            metadata_to_save.custom_metadata = dict(custom_metadata)
           self._blocking_metadata_store.write(
               file_path, serialize_root_metadata(metadata_to_save)
           )
 
       multihost.sync_global_processes(
           multihost.unique_barrier_key(
-              'CheckpointManager:save_metadata',
+              'CheckpointManager:save_root_metadata',
               prefix=self._multiprocessing_options.barrier_sync_key_prefix,
           ),
           processes=self._multiprocessing_options.active_processes,
       )
 
-  def metadata(self) -> Mapping[str, Any]:
-    """See superclass documentation."""
-    if self._metadata is None:
+  def _get_step_metadata(self, step: int) -> StepMetadata:
+    infos = [info for info in self._checkpoints if info.step == step]
+    if not infos:
+      metrics = None
+    else:
+      if len(infos) > 1:
+        logging.warning(
+            'Multiple CheckpointInfos found for step %d. Using the first one.',
+            step,
+        )
+      metrics = infos[0].metrics
+
+    step_metadata = self._checkpointer.metadata(
+        self._get_read_step_directory(step, self.directory),
+    )
+
+    self._default_item.set_if_none(
+        _determine_default_item_mode_from_directory(
+            self._get_read_step_directory(step, self.directory)
+        )
+    )
+    step_metadata.item_metadata = self._maybe_get_default_item(
+        step_metadata.item_metadata
+    )
+
+    if metrics is not None:
+      validated_metrics = step_metadata_serialization.deserialize(
+          {}, metrics=dict(metrics)
+      ).metrics
+      step_metadata = dataclasses.replace(
+          step_metadata,
+          metrics=validated_metrics,
+      )
+
+    return step_metadata
+
+  def _get_root_metadata(self) -> RootMetadata:
+    if self._root_metadata.get().custom_metadata is None:
       if self._metadata_dir.exists():
-        file_path = self._metadata_file_path()
+        file_path = self._root_metadata_file_path()
         if not file_path.exists():
           logging.warning(
-              'New metadata file not found in directory: %s. '
-              'Will try to read legacy metadata file.',
+              'New root metadata file not found in directory: %s. '
+              'Will try to read legacy file.',
               self._metadata_dir,
           )
-          file_path = self._metadata_file_path(legacy=True)
+          file_path = self._root_metadata_file_path(legacy=True)
         serialized_metadata = self._blocking_metadata_store.read(file_path)
-        self._metadata = deserialize_root_metadata(
-            serialized_metadata
-        ).custom_metadata
-        if self._metadata is None:
-          raise FileNotFoundError(f'Failed to read metadata from {file_path}.')
+        if serialized_metadata is None:
+          raise IOError(f'Failed to read metadata from {file_path}')
+        self._root_metadata.set(
+            root_metadata_serialization.deserialize(serialized_metadata)
+        )
       else:
-        self._metadata = {}
-    return self._metadata
+        self._root_metadata.get().custom_metadata = {}
+    return self._root_metadata.get()
+
+  @overload
+  def metadata(self, step: None = None) -> RootMetadata:
+    ...
+
+  @overload
+  def metadata(self, step: int) -> StepMetadata:
+    ...
+
+  def metadata(self, step: int | None = None) -> RootMetadata | StepMetadata:
+    """See superclass documentation."""
+    if step is not None:
+      return self._get_step_metadata(step)
+    return self._get_root_metadata()
 
   def _sort_checkpoints_by_metrics(
       self, checkpoints: checkpoint_info.CheckpointInfos
@@ -1734,85 +1936,74 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
       self._checkpointer.wait_until_finished()  # pytype: disable=attribute-error
 
   def wait_until_finished(self):
-    """See superclass documentation."""
-    process_index = multihost.process_index()
-    logging.info(
-        '[process=%s][thread=%s][wait_until_finished] Initiating wait for Save'
-        ' Finalize thread.',
-        process_index,
-        threading.current_thread().name,
-    )
-    with self._finalize_thread_lock:
-      if self._finalize_thread is None:
-        logging.info(
-            '[process=%s][thread=%s][wait_until_finished] No Save Finalize'
-            ' thread to wait for. Returning.',
-            process_index,
-            threading.current_thread().name,
-        )
-        return
+    """Blocks until any incomplete save operations are completed.
 
-      step = self._finalize_thread.step()
-      try:
-        logging.info(
-            '[process=%s][thread=%s][step=%s][wait_until_finished] Waiting for'
-            ' Save Finalize thread (%s) to complete.',
-            process_index,
-            threading.current_thread().name,
-            step,
-            self._finalize_thread.name,
-        )
-        self._finalize_thread.join()
-        logging.info(
-            '[process=%s][thread=%s][step=%s][wait_until_finished] Done'
-            ' waiting for Save Finalize thread (%s) running at step=%d.',
-            process_index,
-            threading.current_thread().name,
-            step,
-            self._finalize_thread.name,
-            step,
-        )
-      except BaseException:  # pylint:disable=broad-exception-caught
-        logging.exception(
-            '[process=%s][thread=%s][step=%s][wait_until_finished] Save'
-            ' Finalize thread (%s) failed.',
-            process_index,
-            threading.current_thread().name,
-            step,
-            self._finalize_thread.name,
-        )
-        # Only thread which requested save is allowed to clean up.
-        if threading.current_thread() is self._finalize_thread.save_thread():
-          # If an exception occurred in the finalization of the previous
-          # save, we clean up since that checkpoint was never actually saved.
-          latest = self._checkpoints.latest()
-          assert latest is not None
-          assert latest.step == step
-          self._checkpoints.delete_if(lambda info: info.step == step)
-        raise
-      finally:
-        # Only thread which requested save is allowed to reset Save Finalize
-        # thread.
-        if threading.current_thread() is self._finalize_thread.save_thread():
-          logging.info(
-              '[process=%s][thread=%s][step=%s][wait_until_finished] Resetting'
-              ' Save Finalize thread (%s) running at step=%d, also errors if'
-              ' any.',
-              process_index,
-              threading.current_thread().name,
-              step,
-              self._finalize_thread.name,
-              step,
-          )
-          self._finalize_thread = None
+    Note that this method will typically be a no-op if all checkpointers are
+    synchronous, since old checkpoints are already cleaned up immediately after
+    completing `save`, and there is no background thread to wait for.
+
+    If some checkpointers are of type AsyncCheckpointer, however, this method
+    will wait until each of these checkpointers is finished.
+    """
+    process_index = multihost.process_index()
+    current_thread = threading.current_thread()
+
+    if self._finalize_thread.map(
+        lambda t: t is None or (not t.is_alive() and t.exception is None)
+    ):
+      logging.info(
+          '[process=%s][thread=%s][wait_until_finished] No Save Finalize'
+          ' thread to wait for. Returning.',
+          process_index,
+          current_thread.name,
+      )
+      return
+
+    step = self._finalize_thread.get_not_none().step()
+    finalize_thread_name = self._finalize_thread.get_not_none().name
+    try:
+      logging.info(
+          '[process=%s][thread=%s][step=%s][wait_until_finished] Waiting for'
+          ' Save Finalize thread (%s) to complete.',
+          process_index,
+          current_thread.name,
+          step,
+          finalize_thread_name,
+      )
+      # Let all threads join and wait for the finalize thread to complete.
+      # Don't call join() with a lock otherwise we will end up serializing the
+      # access to the finalize thread.
+      self._finalize_thread.get_not_none().join()
+      logging.info(
+          '[process=%s][thread=%s][step=%s][wait_until_finished] Done'
+          ' waiting for Save Finalize thread (%s) running at step=%d.',
+          process_index,
+          current_thread.name,
+          step,
+          finalize_thread_name,
+          step,
+      )
+    except BaseException:  # pylint:disable=broad-exception-caught
+      logging.exception(
+          '[process=%s][thread=%s][step=%s][wait_until_finished] Save'
+          ' Finalize thread (%s) failed.',
+          process_index,
+          current_thread.name,
+          step,
+          finalize_thread_name,
+      )
+      self._checkpoints.delete_if(lambda info: info.step == step)
+      raise
 
   def is_saving_in_progress(self) -> bool:
     """Returns whether a checkpoint save is in progress."""
-    with self._is_saving_in_progress_lock:
-      return self._is_saving_in_progress
+    return self._finalize_thread.map(lambda t: t is not None and t.is_alive())
 
   def check_for_errors(self):
-    """See superclass documentation."""
+    """Checks for any outstanding errors in completed asynchronous save operations.
+
+    Delegates to underlying Checkpointer.
+    """
     if is_async_checkpointer(self._checkpointer):
       self._checkpointer.check_for_errors()  # pytype: disable=attribute-error
 
@@ -1856,6 +2047,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
   def _finalize(self, step: int, steps_to_remove: List[int]):
     """Finalizes individual items and starts garbage collection."""
     process_index = multihost.process_index()
+    current_thread = threading.current_thread()
     self._non_blocking_metadata_store.wait_until_finished()
     self._wait_for_checkpointers()
     # If an error is encountered while waiting for commit futures to complete,
@@ -1871,7 +2063,7 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         '[process=%s][thread=%s][step=%s] CheckpointManager Save Finalize is'
         ' syncing with other hosts...',
         process_index,
-        threading.current_thread().name,
+        current_thread.name,
         step,
     )
     barrier_sync_fn = self._create_thread_safe_barrier_sync_fn()
@@ -1886,14 +2078,12 @@ class CheckpointManager(AbstractCheckpointManager, epy.ContextManager):
         '[process=%s][thread=%s][step=%s] CheckpointManager Save Finalize is'
         ' done on all hosts.',
         process_index,
-        threading.current_thread().name,
+        current_thread.name,
         step,
     )
-    with self._is_saving_in_progress_lock:
-      self._is_saving_in_progress = False
 
   def close(self):
-    """See superclass documentation."""
+    """Waits for outstanding operations to finish and closes internal objects."""
     self.wait_until_finished()
     self._checkpointer.close()
     # Call after checkpointer.close().
