@@ -28,7 +28,7 @@ from etils import epath
 import jax
 import numpy as np
 import optax
-from orbax.checkpoint import options as options_lib
+from orbax.checkpoint import args as args_lib
 from orbax.checkpoint import test_utils
 from orbax.checkpoint import utils
 from orbax.checkpoint._src.handlers import pytree_checkpoint_handler
@@ -36,7 +36,7 @@ from orbax.checkpoint._src.multihost import multihost
 from orbax.checkpoint._src.multihost import multislice
 from orbax.checkpoint.experimental.emergency import checkpoint_manager
 from orbax.checkpoint.experimental.emergency import mesh_consistency
-from orbax.checkpoint.path import step as step_lib
+from orbax.checkpoint.experimental.emergency import process_metadata_checkpoint_handler
 
 PyTree = Any
 PyTreeCheckpointHandler = pytree_checkpoint_handler.PyTreeCheckpointHandler
@@ -86,8 +86,44 @@ class LocalCheckpointManagerTestBase:
   class Test(parameterized.TestCase):
     """Test case."""
 
-    def make_global_mesh(self) -> jax.sharding.Mesh:
-      raise NotImplementedError()
+    def make_global_mesh(
+        self, replica_axis_index: int = 0
+    ) -> jax.sharding.Mesh:
+      """Creates a global mesh for testing purposes.
+
+      This method sets up a global mesh with 8 devices across 4 processes,
+      where each process has 2 local devices. The mesh is constructed based
+      on provided slice processes and a specified replica axis.
+
+      Args:
+        replica_axis_index: The index of the replica axis in the resulting mesh.
+          Must be either 0 or 1. Defaults to 0.
+
+      Returns:
+        A `jax.sharding.Mesh` object representing the fake global mesh.
+
+      Raises:
+        ValueError: If `replica_axis_index` is not 0 or 1.
+      """
+      if replica_axis_index not in [0, 1]:
+        raise ValueError(
+            'replica_axis_index must be 0 or 1 for this test. Got:'
+            f' {replica_axis_index}.'
+        )
+      self.assertEqual(jax.device_count(), 8)
+      self.assertEqual(jax.process_count(), 4)
+      self.assertEqual(jax.local_device_count(), 2)
+
+      # setup global mesh info for 2-slice tests
+      slice_processes = [{0, 1}, {2, 3}]
+      mesh = test_utils.get_fake_global_mesh_for_slices(
+          slice_processes, replica_axis_index
+      )
+      if replica_axis_index == 0:
+        assert mesh.devices.shape == (2, 4), mesh.devices.shape
+      if replica_axis_index == 1:
+        assert mesh.devices.shape == (4, 2), mesh.devices.shape
+      return mesh
 
     def setUp(self):
       super().setUp()
@@ -112,11 +148,6 @@ class LocalCheckpointManagerTestBase:
       self.mesh_tree = mesh_tree
       self.axes_tree = axes_tree
 
-      self.persistent_checkpoint_handler = PyTreeCheckpointHandler(
-          use_ocdbt=True,
-          use_zarr3=True,
-      )
-
       # make sure each process is working on different directories
       self.local_directory = epath.Path(
           self.create_tempdir(
@@ -157,13 +188,26 @@ class LocalCheckpointManagerTestBase:
           options=options,
       )
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
 
       manager.wait_until_finished()
 
-      restored = manager.restore(0, args=PyTreeRestoreArgs())
-
-      test_utils.assert_tree_equal(self, self.pytree, restored)
+      restored = manager.restore(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeRestoreArgs(),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataRestoreArgs(),
+          ),
+      )
+      test_utils.assert_tree_equal(self, self.pytree, restored.state)
 
     def test_local_auto_cleanup(self):
       """Test case."""
@@ -183,19 +227,47 @@ class LocalCheckpointManagerTestBase:
           options=options,
       )
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
-      manager.save(1, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       # step 0 should have been removed
       with self.assertRaisesRegex(ValueError, 'No step path found'):
-        manager.restore(0, args=PyTreeRestoreArgs())
+        manager.restore(
+            0,
+            args=args_lib.Composite(
+                state=PyTreeRestoreArgs(),
+                process_metadata=process_metadata_checkpoint_handler.ProcessMetadataRestoreArgs(),
+            ),
+        )
 
-      restored = manager.restore(1, args=PyTreeRestoreArgs())
+      restored = manager.restore(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeRestoreArgs(),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataRestoreArgs,
+          ),
+      )
 
-      test_utils.assert_tree_equal(self, self.doubled_pytree, restored)
+      test_utils.assert_tree_equal(self, self.doubled_pytree, restored.state)
 
     def test_default_max_to_keep_is_1(self):
       """Test case."""
@@ -213,13 +285,32 @@ class LocalCheckpointManagerTestBase:
           global_mesh=self.global_mesh,
           options=options,
       )
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
-      manager.save(1, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       self.assertEqual(manager.all_steps(), [1])
       self.assertFalse((self.local_directory / '0').exists())
-      self.assertTrue((self.local_directory / '1').exists())
+      self.assertTrue((self.local_directory / '1' / 'state').exists())
+      self.assertTrue(
+          (self.local_directory / '1' / 'process_metadata').exists()
+      )
 
     @parameterized.parameters(
         (False),
@@ -241,9 +332,33 @@ class LocalCheckpointManagerTestBase:
           options=options,
       )
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
-      manager.save(1, args=PyTreeSaveArgs(self.pytree))
-      manager.save(2, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          2,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       # 3 steps should exist
@@ -252,7 +367,7 @@ class LocalCheckpointManagerTestBase:
         self.assertTrue((self.local_directory / str(i)).exists())
         self.assertTrue(
             mesh_consistency.process_metadata_folder(
-                self.local_directory, i
+                self.local_directory / str(i) / 'process_metadata'
             ).exists()
         )
 
@@ -275,29 +390,28 @@ class LocalCheckpointManagerTestBase:
       self.assertEqual(manager2.all_steps(), [2])
       self.assertFalse((self.local_directory / '0').exists())
       self.assertFalse((self.local_directory / '1').exists())
-      self.assertTrue((self.local_directory / '2').exists())
-      self.assertFalse(
-          mesh_consistency.process_metadata_folder(
-              self.local_directory, 0
-          ).exists()
-      )
-      self.assertFalse(
-          mesh_consistency.process_metadata_folder(
-              self.local_directory, 1
-          ).exists()
-      )
+      self.assertTrue((self.local_directory / '2' / 'state').exists())
       self.assertTrue(
-          mesh_consistency.process_metadata_folder(
-              self.local_directory, 2
-          ).exists()
+          (self.local_directory / '2' / 'process_metadata').exists()
       )
 
       # new step save should work
-      manager2.save(3, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager2.save(
+          3,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager2.wait_until_finished()
       self.assertEqual(manager2.all_steps(), [3])
       self.assertFalse((self.local_directory / '2').exists())
-      self.assertTrue((self.local_directory / '3').exists())
+      self.assertTrue((self.local_directory / '3' / 'state').exists())
+      self.assertTrue(
+          (self.local_directory / '3' / 'process_metadata').exists()
+      )
 
     @parameterized.parameters(
         (False),
@@ -322,14 +436,44 @@ class LocalCheckpointManagerTestBase:
           options=options,
       )
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
-      manager.save(1, args=PyTreeSaveArgs(self.pytree))
-      manager.save(2, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          2,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
       self.assertFalse((self.local_directory / '0').exists())
-      self.assertTrue((self.local_directory / '1').exists())
+      self.assertTrue((self.local_directory / '1' / 'state').exists())
+      self.assertTrue(
+          (self.local_directory / '1' / 'process_metadata').exists()
+      )
 
-      self.assertTrue((self.local_directory / '2').exists())
+      self.assertTrue((self.local_directory / '2' / 'state').exists())
+      self.assertTrue(
+          (self.local_directory / '2' / 'process_metadata').exists()
+      )
 
       if multihost.process_index() == 0:
         test_utils.empty_directory(self.local_directory)
@@ -340,29 +484,74 @@ class LocalCheckpointManagerTestBase:
         self.assertFalse((self.local_directory / '1').exists())
         self.assertFalse((self.local_directory / '2').exists())
       else:
-        self.assertTrue((self.local_directory / '1').exists())
-        self.assertTrue((self.local_directory / '2').exists())
+        self.assertTrue((self.local_directory / '1' / 'state').exists())
+        self.assertTrue(
+            (self.local_directory / '1' / 'process_metadata').exists()
+        )
+        self.assertTrue((self.local_directory / '2' / 'state').exists())
+        self.assertTrue(
+            (self.local_directory / '2' / 'process_metadata').exists()
+        )
 
       # subsequent save should be successful and gc should work
-      manager.save(3, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          3,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       if multihost.process_index() == 0:
         self.assertFalse((self.local_directory / '2').exists())
-        self.assertTrue((self.local_directory / '3').exists())
+        self.assertTrue((self.local_directory / '3' / 'state').exists())
+        self.assertTrue(
+            (self.local_directory / '3' / 'process_metadata').exists()
+        )
       else:
         self.assertFalse((self.local_directory / '1').exists())
-        self.assertTrue((self.local_directory / '2').exists())
-        self.assertTrue((self.local_directory / '3').exists())
+        self.assertTrue((self.local_directory / '2' / 'state').exists())
+        self.assertTrue(
+            (self.local_directory / '2' / 'process_metadata').exists()
+        )
+        self.assertTrue((self.local_directory / '3' / 'state').exists())
+        self.assertTrue(
+            (self.local_directory / '3' / 'process_metadata').exists()
+        )
 
-      manager.save(4, args=PyTreeSaveArgs(self.doubled_pytree))
-      manager.save(5, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          4,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          5,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       self.assertFalse((self.local_directory / '2').exists())
       self.assertFalse((self.local_directory / '3').exists())
-      self.assertTrue((self.local_directory / '4').exists())
-      self.assertTrue((self.local_directory / '5').exists())
+      self.assertTrue((self.local_directory / '4' / 'state').exists())
+      self.assertTrue(
+          (self.local_directory / '4' / 'process_metadata').exists()
+      )
+      self.assertTrue((self.local_directory / '5' / 'state').exists())
+      self.assertTrue(
+          (self.local_directory / '5' / 'process_metadata').exists()
+      )
 
     @parameterized.parameters(
         ([[], [], [], []], {0: {}, 1: {}}),
@@ -405,13 +594,37 @@ class LocalCheckpointManagerTestBase:
           options=options,
       )
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
-      manager.save(1, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
-      manager.save(2, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          2,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       if multihost.process_index() in processes_to_clear:
@@ -464,10 +677,26 @@ class LocalCheckpointManagerTestBase:
           options=options,
       )
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
-      manager.save(1, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       if multihost.process_index() in empty_process:
@@ -475,7 +704,15 @@ class LocalCheckpointManagerTestBase:
 
       manager.reload()
 
-      manager.save(2, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          2,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       self.assertEqual(manager.all_steps(), [1, 2] if keep else [2])
@@ -512,8 +749,24 @@ class LocalCheckpointManagerTestBase:
           options=options,
       )
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
-      manager.save(1, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       # confirm all hosts have latest step as 1
@@ -554,7 +807,15 @@ class LocalCheckpointManagerTestBase:
       )
       self.assertEmpty(manager._steps)
 
-      manager.save(0, args=PyTreeSaveArgs(self.pytree))
+      manager.save(
+          0,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       # confirm all hosts have latest step as 0
@@ -572,9 +833,33 @@ class LocalCheckpointManagerTestBase:
       self.assertIsNone(manager.latest_step())
       self.assertEqual(manager._steps, [])
 
-      manager.save(1, args=PyTreeSaveArgs(self.doubled_pytree))
-      manager.save(2, args=PyTreeSaveArgs(self.pytree))
-      manager.save(3, args=PyTreeSaveArgs(self.doubled_pytree))
+      manager.save(
+          1,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          2,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
+      manager.save(
+          3,
+          args=args_lib.Composite(
+              state=PyTreeSaveArgs(self.doubled_pytree),
+              process_metadata=process_metadata_checkpoint_handler.ProcessMetadataSaveArgs(
+                  global_mesh=self.global_mesh
+              ),
+          ),
+      )
       manager.wait_until_finished()
 
       if multihost.process_index() == 0:
@@ -630,10 +915,6 @@ class CheckpointManagerTestBase:
           self.persistent_directory,
       )
 
-      self.persistent_checkpoint_handler = PyTreeCheckpointHandler(
-          use_ocdbt=True,
-          use_zarr3=True,
-      )
       test_utils.set_tensorstore_driver_for_test()
       test_utils.sync_global_processes('CheckpointManagerTest:setup_complete')
 
@@ -643,9 +924,8 @@ class CheckpointManagerTestBase:
           'CheckpointManagerTest:teardown_complete'
       )
 
-    def setup_pytree(self, replica_axis_index: int = 0):
+    def setup_pytree(self, global_mesh: jax.sharding.Mesh):
       """Create mesh and pytree."""
-      global_mesh = self.make_global_mesh(replica_axis_index)
       pytree = {
           'a': test_utils.create_sharded_array(
               np.arange(8), global_mesh, jax.sharding.PartitionSpec(None)
@@ -681,7 +961,7 @@ class CheckpointManagerTestBase:
         expectation: bool,
     ):
       """Test case."""
-      global_mesh, pytree = self.setup_pytree()
+      global_mesh, pytree = self.setup_pytree(self.make_global_mesh())
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       options = CheckpointManagerOptions(
           local=LocalCheckpointOptions(
@@ -720,28 +1000,14 @@ class CheckpointManagerTestBase:
     )
     def test_global_max(self, inputs, expectation):
       """Test case."""
-      global_mesh, pytree = self.setup_pytree()
-      abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
-      options = CheckpointManagerOptions(
-          local=LocalCheckpointOptions(save_interval_steps=1, max_to_keep=1),
-          persistent=PersistentCheckpointOptions(
-              save_interval_steps=1, max_to_keep=4
-          ),
-      )
-      manager = CheckpointManager(
-          local_directory=self.local_directory,
-          persistent_directory=self.persistent_directory,
-          global_mesh=global_mesh,
-          abstract_state=abstract_state,
-          options=options,
-      )
-
       local_host_inputs = inputs[multihost.process_index()]
       if not isinstance(local_host_inputs, list):
         local_host_inputs = [local_host_inputs]
         expectation = [expectation]
       self.assertEqual(
-          manager._global_max(local_host_inputs),
+          checkpoint_manager._global_max(
+              local_host_inputs, checkpoint_manager._get_global_broadcast_fn()
+          ),
           expectation,
       )
 
@@ -761,7 +1027,7 @@ class CheckpointManagerTestBase:
         expectation,
     ):
       """Test case."""
-      global_mesh, pytree = self.setup_pytree()
+      global_mesh, pytree = self.setup_pytree(self.make_global_mesh())
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       options = CheckpointManagerOptions(
           local=LocalCheckpointOptions(
@@ -811,7 +1077,7 @@ class CheckpointManagerTestBase:
         total_steps: Optional[int] = 4,
     ):
       """Test case."""
-      global_mesh, pytree = self.setup_pytree()
+      global_mesh, pytree = self.setup_pytree(self.make_global_mesh())
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       options = CheckpointManagerOptions(
           local=LocalCheckpointOptions(
@@ -852,7 +1118,7 @@ class CheckpointManagerTestBase:
     )
     def test_save(self, enable_async_checkpointing: bool):
       """Test case."""
-      global_mesh, pytree = self.setup_pytree()
+      global_mesh, pytree = self.setup_pytree(self.make_global_mesh())
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       options = CheckpointManagerOptions(
           local=LocalCheckpointOptions(save_interval_steps=3, max_to_keep=2),
@@ -873,26 +1139,7 @@ class CheckpointManagerTestBase:
         manager.save(i, args=PyTreeSaveArgs(pytree))
       manager.wait_until_finished()
 
-      in_primary_slice = multislice.in_slice(
-          multihost.process_index(), global_mesh
-      )
-      if in_primary_slice:
-        persistent_multiprocessing_options = options_lib.MultiprocessingOptions(
-            primary_host=global_mesh.devices[0].flat[0].process_index,
-            active_processes=multihost.unique_processes_from_devices(
-                global_mesh.devices[0]
-            ),
-            barrier_sync_key_prefix='persistent',
-        )
-        persistent_manager = manager._make_persistent_checkpoint_manager(
-            persistent_multiprocessing_options
-        )
-        self.assertEmpty(step_lib.checkpoint_steps(self.local_directory))
-        self.assertEqual(persistent_manager.all_steps(), [5, 10, 15])
-      else:
-        local_manager = manager._make_local_checkpoint_manager()
-        self.assertNotEmpty(step_lib.checkpoint_steps(self.local_directory))
-        self.assertEqual(local_manager.all_steps(), [12, 15])
+      self.assertEqual(manager.all_steps(), [5, 10, 12, 15])
 
     @parameterized.parameters(
         (False, 0),
@@ -903,7 +1150,9 @@ class CheckpointManagerTestBase:
         self, enable_async_checkpointing: bool, replica_axis_index: int
     ):
       """Test restore successfully."""
-      global_mesh, pytree = self.setup_pytree(replica_axis_index)
+      global_mesh, pytree = self.setup_pytree(
+          self.make_global_mesh(replica_axis_index)
+      )
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       pytree_double = test_utils.apply_function(pytree, lambda x: x * 2)
       options = CheckpointManagerOptions(
@@ -916,7 +1165,7 @@ class CheckpointManagerTestBase:
       )
 
       with mock.patch.object(
-          CheckpointManager,
+          checkpoint_manager._MultisliceCheckpointManager,
           '_restore_from_persistent',
           autospec=True,
           return_value=None,
@@ -945,7 +1194,9 @@ class CheckpointManagerTestBase:
     )
     def test_slice_setup(self, replica_axis_index: int):
       """Test restoration after slice swap (via global mesh)."""
-      global_mesh, pytree = self.setup_pytree(replica_axis_index)
+      global_mesh, pytree = self.setup_pytree(
+          self.make_global_mesh(replica_axis_index)
+      )
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       pytree_double = test_utils.apply_function(pytree, lambda x: x * 2)
       options = CheckpointManagerOptions(
@@ -963,8 +1214,10 @@ class CheckpointManagerTestBase:
           options=options,
           abstract_state=abstract_state,
       ) as manager:
-        self.assertEqual(manager._persistent_primary_host, 0)
-        self.assertEqual(manager._local_primary_host, 2)
+        self.assertEqual(
+            manager._checkpoint_manager._persistent_primary_host, 0
+        )
+        self.assertEqual(manager._checkpoint_manager._local_primary_host, 2)
         slice_id = multislice.process_slice_id(
             multihost.process_index(),
             global_mesh,
@@ -1034,7 +1287,9 @@ class CheckpointManagerTestBase:
     )
     def test_slice_swap_check_steps(self, replica_axis_index: int):
       """Test step detection across slice swap (via mesh)."""
-      global_mesh, pytree = self.setup_pytree(replica_axis_index)
+      global_mesh, pytree = self.setup_pytree(
+          self.make_global_mesh(replica_axis_index)
+      )
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       options = CheckpointManagerOptions(
           local=LocalCheckpointOptions(save_interval_steps=1, max_to_keep=10),
@@ -1051,8 +1306,10 @@ class CheckpointManagerTestBase:
           options=options,
           abstract_state=abstract_state,
       ) as manager:
-        self.assertEqual(manager._persistent_primary_host, 0)
-        self.assertEqual(manager._local_primary_host, 2)
+        self.assertEqual(
+            manager._checkpoint_manager._persistent_primary_host, 0
+        )
+        self.assertEqual(manager._checkpoint_manager._local_primary_host, 2)
         slice_id = multislice.process_slice_id(
             multihost.process_index(),
             global_mesh,
@@ -1091,8 +1348,10 @@ class CheckpointManagerTestBase:
           options=options,
           abstract_state=abstract_state,
       ) as manager:
-        self.assertEqual(manager._persistent_primary_host, 2)
-        self.assertEqual(manager._local_primary_host, 0)
+        self.assertEqual(
+            manager._checkpoint_manager._persistent_primary_host, 2
+        )
+        self.assertEqual(manager._checkpoint_manager._local_primary_host, 0)
         slice_id = multislice.process_slice_id(
             multihost.process_index(),
             new_global_mesh,
@@ -1119,7 +1378,9 @@ class CheckpointManagerTestBase:
         self, enable_async_checkpointing: bool, replica_axis_index: int
     ):
       """Test restore successfully."""
-      global_mesh, pytree = self.setup_pytree(replica_axis_index)
+      global_mesh, pytree = self.setup_pytree(
+          self.make_global_mesh(replica_axis_index)
+      )
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       pytree_double = test_utils.apply_function(pytree, lambda x: x * 2)
       options = CheckpointManagerOptions(
@@ -1171,7 +1432,9 @@ class CheckpointManagerTestBase:
         self, enable_async_checkpointing: bool, replica_axis_index: int
     ):
       """Test case."""
-      global_mesh, pytree = self.setup_pytree(replica_axis_index)
+      global_mesh, pytree = self.setup_pytree(
+          self.make_global_mesh(replica_axis_index)
+      )
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       pytree_double = test_utils.apply_function(pytree, lambda x: x * 2)
       options = CheckpointManagerOptions(
@@ -1203,7 +1466,7 @@ class CheckpointManagerTestBase:
 
     def test_process_index_metadata(self):
       """Test case."""
-      global_mesh, pytree = self.setup_pytree()
+      global_mesh, pytree = self.setup_pytree(self.make_global_mesh())
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       with CheckpointManager(
           local_directory=self.local_directory,
@@ -1212,7 +1475,7 @@ class CheckpointManagerTestBase:
           abstract_state=abstract_state,
       ) as mngr:
         metadata_folder = mesh_consistency.process_metadata_folder(
-            self.local_directory, 0
+            self.local_directory / '0' / 'process_metadata'
         )
         self.assertFalse(metadata_folder.exists())
 
@@ -1255,7 +1518,7 @@ class CheckpointManagerTestBase:
         return step in persistent_expectation
 
       max_to_keep = 10
-      global_mesh, pytree = self.setup_pytree()
+      global_mesh, pytree = self.setup_pytree(self.make_global_mesh())
       abstract_state = jax.tree.map(utils.to_shape_dtype_struct, pytree)
       options = CheckpointManagerOptions(
           local=LocalCheckpointOptions(
@@ -1301,6 +1564,8 @@ class CheckpointManagerTestBase:
         def __init__(self, devices, axis_names):
           self.devices = devices
           self.axis_names = axis_names
+          self.shape = mesh_shape
+          self.shape_tuple = tuple(mesh_shape)
 
       devices = np.arange(np.prod(mesh_shape)).reshape(mesh_shape)
       with mock.patch.object(jax.sharding, 'Mesh', wraps=FakeMesh):
