@@ -18,6 +18,8 @@
 
 from orbax.checkpoint._src.path import async_utils
 from orbax.checkpoint._src.path import atomicity
+from orbax.checkpoint._src.path import atomicity_defaults
+from orbax.checkpoint._src.path import atomicity_types
 from orbax.checkpoint._src.path import deleter
 from orbax.checkpoint._src.path import format_utils
 from orbax.checkpoint._src.path import step
