@@ -50,6 +50,7 @@ from orbax.checkpoint._src.serialization import tensorstore_utils as ts_utils
 from orbax.checkpoint._src.serialization import type_handlers
 from orbax.checkpoint._src.tree import utils as tree_utils
 
+
 PyTree = Any
 
 
@@ -203,15 +204,31 @@ def apply_function(tree, function):
 
 def assert_array_equal(testclass, v_expected, v_actual):
   """Asserts that two arrays are equal."""
+  if hasattr(v_expected, 'dtype'):
+    testclass.assertEqual(v_expected.dtype, v_actual.dtype)
   testclass.assertIsInstance(v_actual, type(v_expected))
   if isinstance(v_expected, jax.Array):
-    testclass.assertEqual(
-        len(v_expected.addressable_shards), len(v_actual.addressable_shards)
-    )
-    for shard_expected, shard_actual in zip(
-        v_expected.addressable_shards, v_actual.addressable_shards
-    ):
-      np.testing.assert_array_equal(shard_expected.data, shard_actual.data)
+    if jax.dtypes.issubdtype(v_expected.dtype, jax.dtypes.prng_key):
+      # random key
+      testclass.assertTrue(
+          jax.dtypes.issubdtype(v_actual.dtype, jax.dtypes.prng_key)
+      )
+      expected_array = jax.random.key_data(v_expected)
+      actual_array = jax.random.key_data(v_actual)
+      assert_array_equal(testclass, expected_array, actual_array)
+      testclass.assertEqual(
+          str(jax.random.key_impl(v_expected)),
+          str(jax.random.key_impl(v_actual)),
+      )
+    else:
+      # regular array
+      testclass.assertEqual(
+          len(v_expected.addressable_shards), len(v_actual.addressable_shards)
+      )
+      for shard_expected, shard_actual in zip(
+          v_expected.addressable_shards, v_actual.addressable_shards
+      ):
+        np.testing.assert_array_equal(shard_expected.data, shard_actual.data)
   elif isinstance(v_expected, (np.ndarray, jnp.ndarray)):
     np.testing.assert_array_equal(v_expected, v_actual)
   else:
@@ -569,6 +586,10 @@ def _get_test_wrapper_function(test_func):
 def barrier_compatible_test(cls):
   """A decorator to be used with a test class.
 
+  This is primarily needed when different processes in a multihost test may be
+  executing different code. This will cause operation IDs to get out of sync. If
+  all processes always execute the same code, this decorator is not needed.
+
   E.g.
 
   @barrier_compatible_test
@@ -604,13 +625,15 @@ def ensure_atomic_save(
         temp_ckpt_dir,
         final_ckpt_dir,
         checkpoint_metadata_store=metadata_store,
-    ).finalize()
+    ).finalize(
+    )
   else:
     atomicity.AtomicRenameTemporaryPath(
         temp_ckpt_dir,
         final_ckpt_dir,
         checkpoint_metadata_store=metadata_store,
-    ).finalize()
+    ).finalize(
+    )
 
 
 def create_single_replica_restore_args(
@@ -729,7 +752,7 @@ def assert_every_n_is_x_apart(testclass, values, n, x):
 
 
 def get_expected_chunk_shape(
-    arr: jax.Array, *, use_replica_parallel: bool = False
+    arr: jax.Array, *, use_replica_parallel: bool = True
 ) -> tuple[int, ...]:
   """Expected chunk shape for an array, accounting for replica-parallel."""
   if not use_replica_parallel:
